#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/6/11 15:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *
from meutils.io.files_utils import to_url
from meutils.decorators.retry import retrying
from meutils.llm.clients import AsyncClient
from meutils.schemas.openai_types import CompletionRequest
from meutils.schemas.video_types import VideoRequest

from meutils.notice.feishu import send_message_for_volc as send_message

from meutils.db.redis_db import redis_aclient
from meutils.llm.check_utils import check_token_for_volc as check
from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status, BackgroundTasks

# Please activate
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=8W6kk8"  # 超刷


async def get_valid_token(tokens: Optional[list] = None):
    tokens = tokens or await get_series(FEISHU_URL)

    for token in tokens:
        if await check(token):

            logger.debug(f"有效 {token}")

            return token
        else:
            logger.debug(f"无效 {token}")
    _ = f"{time.ctime()}\n\n{FEISHU_URL}\n\n所有token无效"
    logger.error(_)
    send_message(_, n=3)


# check_image_and_video = partial(check, purpose='video and image')


@retrying(max_retries=5)
async def create_task(request: Union[CompletionRequest, VideoRequest], api_key: Optional[str] = None):
    # api_key = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, check_token=check)
    # api_key = api_key or await get_valid_token()

    feishu_url = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=rcoDg7"
    api_key = api_key or await get_next_token_for_polling(
        feishu_url=feishu_url,
        from_redis=True,
        ttl=24 * 3600,
        check_token=check
    )

    logger.debug(f"api_key: {api_key}")
    if isinstance(request, VideoRequest):  # 兼容jimeng
        request.prompt = f"{request.prompt} --duration {request.duration}"

        payload = {
            "model": "doubao-seedance-1-0-lite-t2v-250428",

            "content": [
                {
                    "type": "text",
                    "text": request.prompt
                }
            ]
        }
        if request.image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        }
                    }
                ]
            }
        if request.image_url and request.tail_image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        },
                        "role": "first_frame"
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.tail_image_url
                        },
                        "role": "last_frame"
                    }
                ]
            }

        payload['model'] = "doubao-seedance-1-0-pro-250528"  # 未来注销

    else:

        payload = {
            "model": request.model,
        }

        if hasattr(request, 'content'):
            payload["content"] = request.content

        elif image_urls := request.last_urls.get("image_url"):
            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                },
                {
                    "type": "image_url",
                    "image_url": {
                        "url": image_urls[-1]
                        # "url": await to_url(image_urls[-1], filename=".png")

                    }
                }]
        else:
            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                }]

    logger.debug(payload)

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.post(
        path="/contents/generations/tasks",
        cast_to=object,
        body=payload
    )

    if task_id := response.get('id'):
        await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)

    return response  # {'id': 'cgt-20250611152553-r46ql'}


async def get_task(task_id: str):
    token = await redis_aclient.get(task_id)  # 绑定对应的 token
    api_key = token and token.decode()
    if not token:
        raise HTTPException(status_code=404, detail="TaskID not found")

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.get(
        path=f"/contents/generations/tasks/{task_id}",
        cast_to=object,
    )

    return response


# 执行异步函数
if __name__ == "__main__":
    # api_key = "07139a08-e360-44e2-ba31-07f379bf99ed"  # {'id': 'cgt-20250611164343-w2bzq'} todo 过期调用get

    api_key = "c2449725-f758-42af-8f2c-e05b68dd06ad"  # 欠费

    api_key = None

    request = CompletionRequest(
        model="doubao-seedance-1-0-pro-250528",
        # model="doubao-seaweed-241128",
        messages=[
            {"role": "user",
             "content": "无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验  --resolution 1080p  --duration 5 --camerafixed false"}
        ],
    )
    request = VideoRequest(
        model="doubao-seedance-1-0-pro-250528",
        prompt="无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验",
        duration=10
    )
    # r = arun(create_task(request))
    # r = {'id': 'cgt-20250612172542-6nbt2'}

    # arun(get_task(r.get('id')))

    # arun(get_task("cgt-20250707162431-smhwc"))

    # arun(get_task("cgt-20250707160713-j8kll"))

    arun(get_valid_token())

"""
{'id': 'cgt-20250613160030-2dvd7',
 'model': 'doubao-seedance-1-0-pro-250528',
 'status': 'running',
 'created_at': 1749801631,
 'updated_at': 1749801631}
 
openai.NotFoundError: Error code: 404 - {'error': {'code': 'ModelNotOpen', 'message': 'Your account 2107813928 has not activated the model doubao-seedance-1-0-pro. Please activate the model service in the Ark Console. Request id: 021749718307793fc1441381f8ed755824bb9d58a45ee6cb04068', 'param': '', 'type': 'NotFound'}}


curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer 21289782-f54f-481b-857e-ba1c8b94927b" \
  -d '{
    "model": "doubao-seaweed-241128",
    "content": [
        {
            "type": "text",
            "text": "女孩抱着狐狸，女孩睁开眼，温柔地看向镜头，狐狸友善地抱着，镜头缓缓拉出，女孩的头发被风吹动  --resolution 720p --duration 5"
        },
        {
            "type": "image_url",
            "image_url": {
                "url": "https://s3.ffire.cc/cdn/20250613/vS2jaDCmA8crncXMR4fB7z_.png"
            }
        }
    ]
}'

curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer 05702f67-53ba-438a-a22e-6e6a690b5843" \
  -d '{
    "model": "doubao-seedance-1-0-pro-250528",
    "content": [
        {
            "type": "text",
            "text": "女孩抱着狐狸，女孩睁开眼，温柔地看向镜头，狐狸友善地抱着，镜头缓缓拉出，女孩的头发被风吹动  --resolution 480p --duration 5"
        },
        {
            "type": "image_url",
            "image_url": {
                "url": "https://s3.ffire.cc/cdn/20250613/vS2jaDCmA8crncXMR4fB7z_.png"
            }
        }
    ]
}'

curl -X POST https://ark.cn-beijing.volces.com/api/v3/contents/generations/tasks \
  -H "Content-Type: application/json" \
  -H "Authorization: Bearer e32b1693-147e-40db-a83a-82f85cfa6360" \
  -d '{
    "model": "doubao-seaweed-241128",
    "content": "a dog"
}'


{'id': 'cgt-20250613160030-2dvd7',
 'model': 'doubao-seedance-1-0-pro-250528',
 'status': 'succeeded',
 'content': {'video_url': 'https://ark-content-generation-cn-beijing.tos-cn-beijing.volces.com/doubao-seedance-1-0-pro/02174980163157800000000000000000000ffffac182c17b26890.mp4?X-Tos-Algorithm=TOS4-HMAC-SHA256&X-Tos-Credential=AKLTYjg3ZjNlOGM0YzQyNGE1MmI2MDFiOTM3Y2IwMTY3OTE%2F20250613%2Fcn-beijing%2Ftos%2Frequest&X-Tos-Date=20250613T080120Z&X-Tos-Expires=86400&X-Tos-Signature=5e0928f738f49b93f54923549de4c65940c5007d5e86cb5ebadc756cca3aa03e&X-Tos-SignedHeaders=host'},
 'usage': {'completion_tokens': 246840, 'total_tokens': 246840},
 'created_at': 1749801631,
 'updated_at': 1749801680}
 
 
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/9/24 10:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.io.files_utils import to_bytes

from meutils.apis.kuaishou import klingai

from meutils.schemas.kling_types import STATUSES, send_message
from meutils.schemas.kling_types import ImageRequest, Task, TaskResponse
from meutils.schemas.kuaishou_types import BASE_URL, KlingaiImageRequest
from meutils.schemas.image_types import KlingImageRequest


async def create_task(request: ImageRequest, vip: bool = False):
    url = None
    if request.image:
        file = await to_bytes(request.image)
        file_task = await klingai.upload(file)
        url = file_task.url
        # return url

    logger.debug(request.model_dump_json(exclude_none=True, indent=4))

    request = KlingaiImageRequest(url=url, **request.model_dump())
    task_response = await klingai.create_task_plus(request, vip=vip)

    return task_response

    # {
    #     "code": 0,
    #     "message": "string",
    #     "request_id": "string",
    #     "data": {
    #         "task_id": "string",
    #         "task_status": "string",
    #         "created_at": 0,
    #         "updated_at": 0
    #     }
    # }


async def get_task(task_id: str, token: str, oss: Optional[str] = None):
    vip = "mini" not in task_id
    task_type = "images" if 'image' in task_id else "videos"

    result = await klingai.get_task_plus(task_id, token)

    data = result['data']
    message = result['message']
    status_code = result['status']

    task = None
    try:
        works = data['works']
        urls = [work['resource']['resource'] for work in works]

        items = [
            {
                "id": f"{uuid.uuid4()}",  # work['workId'],
                "url": url,
                "duration": f"{work['resource']['duration'] / 1000:.1f}",
            } for work, url in zip(works, urls)
        ]

        task_result = {task_type: items}
        task = Task(
            task_id=str(data['task']['id']),
            task_status=STATUSES.get(data['status'], data['status']),
            task_status_msg=data['message'],
            task_result=task_result
        )

        if data['status'] not in STATUSES:
            send_message(f"未知状态：{data['status']}")
            send_message(data)

    except Exception as e:
        logger.error(e)
        send_message(f"task_id:{task_id} 失败\n{e}")
        send_message(data)

    response = TaskResponse(
        code=status_code,
        message=message,
        data=task,
    )
    return response


# {
#     "code": 0,
#     "message": "string",
#     "request_id": "string",
#     "data": {
#         "task_id": "string",
#         "task_status": "string",
#         "task_status_msg": "string",
#         "created_at": 1722769557708,
#         "updated_at": 1722769557708,
#         "task_result": {
#             "videos": [
#                 {
#                     "id": "string",
#                     "url": "string",
#                     "duration": "string"
#                 }
#             ]
#         }
#     }
# }


async def generate(request: KlingImageRequest):
    request = ImageRequest(**request.model_dump(exclude_none=True))
    task = await create_task(request, vip=True)

    for i in range(1, 15):
        await asyncio.sleep(15 / i)
        try:
            data = await get_task(task.data.task_id, task.system_fingerprint)
            images = data.data.task_result.get('images', [])
            return {"data": images}

        except Exception as e:
            logger.debug(e)
            continue


if __name__ == '__main__':
    request = ImageRequest(
        prompt="多肉植物，带着水珠，潮玩盲盒风格，皮克斯，3D质感，温馨的环境，丰富的场景，最佳画质，超精细，Octane渲染",
        aspect_ratio="1:1",
        image_fidelity=0.5,
        n=2,
        negative_prompt="",
    )


    # task = arun(create_task(request, vip=True))
    # task = arun(create_task(request, vip=False))

    # image = "https://oss.ffire.cc/files/old.jpg"
    # request = ImageRequest(
    #     prompt="互相拥抱",
    #     image=image
    # )
    # task = arun(create_task(request, vip=False))

    # print(task.model_dump_json(exclude_none=True, indent=4))


    request = KlingImageRequest(
        prompt="多肉植物，带着水珠，潮玩盲盒风格，皮克斯，3D质感，温馨的环境，丰富的场景，最佳画质，超精细，Octane渲染",
        aspect_ratio="1:1",
        image_fidelity=0.5,
        n=2,
        negative_prompt="",
    )
    arun(generate(request))

    # arun(get_task(task.data.task_id, task.system_fingerprint, oss='kling'))
    # arun(get_task(task.data.task_id, task.system_fingerprint))

    # arun(get_task("kling-image-pro-56994904", task.system_fingerprint, oss='kling'))

    # arun(get_task(task.data.task_id, task.system_fingerprint, oss='kling'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : adapters
# @Time         : 2025/5/30 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_url_fal
from meutils.str_utils.json_utils import json_path
from meutils.schemas.openai_types import CompletionRequest
from meutils.schemas.image_types import ImageRequest
from meutils.llm.openai_utils import chat_completion, chat_completion_chunk, create_chat_completion_chunk


async def stream_to_nostream(
        request: CompletionRequest,
):
    pass


async def chat_for_image(
        generate: Callable,
        request: CompletionRequest,
        api_key: Optional[str] = None,
):
    generate = partial(generate, api_key=api_key)

    if not request.stream or request.last_user_content.startswith(  # 跳过nextchat
            (
                    "hi",
                    "使用四到五个字直接返回这句话的简要主题",
                    "简要总结一下对话内容，用作后续的上下文提示 prompt，控制在 200 字以内"
            )):
        chat_completion.choices[0].message.content = "请设置`stream=True`"
        return chat_completion

    prompt = request.last_user_content
    if request.last_urls:  # image_url
        urls = await to_url_fal(request.last_urls["image_url"], content_type="image/png")
        prompt = "\n".join(urls + [prompt])

    request = ImageRequest(
        model=request.model,
        prompt=prompt,
    )

    future_task = asyncio.create_task(generate(request))  # 异步执行

    async def gen():
        for i in f"> 🖌️正在绘画\n\n```json\n{request.model_dump_json(exclude_none=True)}\n```\n\n":
            await asyncio.sleep(0.05)
            yield i

        response = await future_task

        for image in response.data:
            yield f"![{image.revised_prompt}]({image.url})\n\n"

    chunks = create_chat_completion_chunk(gen(), redirect_model=request.model)
    return chunks



async def chat_for_video(
        get_task: Callable,  # response
        taskid: str,
):
    """异步任务"""

    async def gen():

        # 获取任务
        for i in f"""> VideoTask(id={taskid})\n""":
            await asyncio.sleep(0.03)
            yield i

        yield f"[🤫 任务进度]("
        for i in range(60):
            await asyncio.sleep(3)
            response = await get_task(taskid)  # 包含  "status"

            logger.debug(response)
            if response.get("status", "").lower().startswith(("succ", "fail")):

                yield ")🎉🎉🎉\n\n"

                yield f"""```json\n{json.dumps(response, indent=4, ensure_ascii=False)}\n```"""

                if urls := json_path(response, expr='$..[url,image_url,video_url]'):  # 所有url
                    for i, url in enumerate(urls, 1):
                        yield f"\n\n[下载链接{i}]({url})\n\n"

                break

            else:
                yield "🚀"

    chunks = create_chat_completion_chunk(gen())
    return chunks


if __name__ == '__main__':
    request = CompletionRequest(
        model="deepseek-r1-Distill-Qwen-1.5B",
        messages=[
            {"role": "user", "content": "``hi"}
        ],
        stream=False,
    )
    arun(chat_for_image(None, request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : audio
# @Time         : 2025/3/23 16:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from dashscope.audio.asr import Transcription

file_urls = [
    "https://dashscope.oss-cn-beijing.aliyuncs.com/samples/audio/paraformer/hello_world_female2.wav",
    "https://dashscope.oss-cn-beijing.aliyuncs.com/samples/audio/paraformer/hello_world_male2.wav",
]
language_hints = ["zh", "en"]

data = {
    "model": "paraformer-v2",
    "input": {"file_urls": file_urls},
    "parameters": {
        "channel_id": [0],
        "language_hints": language_hints,
        "vocabulary_id": "vocab-Xxxx",
    },
}

task_response = Transcription().async_call(
    model="paraformer-v2",
    file_urls=file_urls,
    language_hints=language_hints,
)

response = Transcription().wait(task_response)  # .output.results



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2024/9/25 15:27
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
# from minio import Minio
from meutils.oss.minio_oss import Minio
import os
from openai import OpenAI

client = OpenAI(
    api_key=os.getenv("DASHSCOPE_API_KEY"),
    base_url="https://dashscope.aliyuncs.com/compatible-mode/v1",
)

file = client.files.create(
    file=open("/Users/betterme/PycharmProjects/AI/docker-compose.yml", "rb"),
    purpose='file-extract'
)
client.files.content(file.id)

if __name__ == '__main__':
    print(file)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : pd_utils
# @Time         : 2020/11/12 11:35 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

from sklearn.model_selection import StratifiedKFold, ShuffleSplit

from meutils.common import *
from meutils.decorators import deprecated


def df2dict(df):
    assert df.shape[1] == 2
    return dict(zip(*map(df.__getitem__, df.columns)))


def pd_bind(cls=pd.DataFrame, *funcs):
    for f in funcs:
        setattr(cls, f.__name__, f)


df_memory = lambda df, deep=False: df.memory_usage(deep=deep).sum() / 1024 ** 2


def df_kfold_split(df, label='target', cv=5, split_seed=42):
    """
        from sklearn.datasets import load_iris
        df = pd.concat(load_iris(return_X_y=True, as_frame=1), 1)
        for df_train, df_valid in df_kfold_split(df, 'target'):
            break

    @param df:
    @param label:
    @param cv:
    @param split_seed:
    @return:
    """
    if label in df:
        folds = StratifiedKFold(n_splits=cv, shuffle=True, random_state=split_seed).split(df, df[label])
    else:
        logger.info(f"{label} not in df")
        folds = ShuffleSplit(n_splits=cv, random_state=split_seed).split(df)  # todo: 兼容时间序列

    for train_index, valid_index in folds:
        yield df.iloc[train_index], df.iloc[valid_index]  # df_train, df_valid


def dfs_join_(dfs):
    return reduce(lambda left, right: pd.merge(left, right), dfs)


def dfs_join(df, dfs, keys_list=None):
    for df_, keys in tqdm(zip(dfs, keys_list), desc='joining🐢'):
        df = df.merge(df_, on=keys)
    return df


def df2json(df, filename='x.json'):
    df.to_json(filename, orient='records', lines=True)


def split(df, num_part=None, batch_size=None):
    assert any((num_part, batch_size)), "num_part, batch_size 不能同时为 None"

    if num_part is None:
        num_part = max(len(df) // batch_size, 1)

    yield from np.array_split(df, num_part)  # 仍保留原始索引


def duplicate_columns(frame):
    """keep='first' 
    https://stackoverflow.com/questions/14984119/python-pandas-remove-duplicate-columns/32961145#32961145
    数据大:
        dups = duplicate_columns(df)
        df.drop(dups, 1)

    数据小:
        df.T.drop_duplicates().T
    """
    frame = frame.fillna(-123456)  # 处理缺失值

    groups = frame.columns.to_series().groupby(frame.dtypes).groups
    dups = []
    for t, v in groups.items():
        dcols = frame[v].to_dict(orient="list")

        vs = list(dcols.values())
        ks = list(dcols.keys())
        lvs = len(vs)

        for i in range(lvs):
            for j in range(i + 1, lvs):
                if vs[i] == vs[j]:
                    dups.append(ks[j])  # keep='first'
                    break
    return dups


def reduce_mem_usage(df):
    numerics = ['int16', 'int32', 'int64', 'float16', 'float32', 'float64']
    start_mem = df.memory_usage().sum() / 1024 ** 2
    for col in tqdm(df.columns, desc="Reduce memory"):
        col_type = df[col].dtypes
        if col_type in numerics:
            c_min = df[col].min()
            c_max = df[col].max()
            if str(col_type)[:3] == 'int':
                if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                    df[col] = df[col].astype(np.int8)
                elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                    df[col] = df[col].astype(np.int16)
                elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                    df[col] = df[col].astype(np.int32)
                elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                    df[col] = df[col].astype(np.int64)
            else:
                if c_min > np.finfo(np.float16).min and c_max < np.finfo(np.float16).max:
                    df[col] = df[col].astype(np.float16)
                elif c_min > np.finfo(np.float32).min and c_max < np.finfo(np.float32).max:
                    df[col] = df[col].astype(np.float32)
                else:
                    df[col] = df[col].astype(np.float64)
        # else:
        #     df[col] = df[col].astype('category')

    end_mem = df.memory_usage().sum() / 1024 ** 2
    print('Memory usage after optimization is: {:.2f} MB'.format(end_mem))
    print('Decreased by {:.1f}%'.format(100 * (start_mem - end_mem) / start_mem))

    return df


def to_html_plus(df, title='Title', subtitle='Subtitle'):
    from jinja2 import Environment, PackageLoader
    env = Environment(loader=PackageLoader('meutils'))
    template = env.get_template('df_html.j2')

    content = template.render(df_to_html=df.to_html(), title=title, subtitle=subtitle)
    # with open('./df.html', 'w') as fp:
    #     fp.write(content)
    return content


def to_compress(df: pd.DataFrame, filename, method='zip', archive_name='data.csv', index=False, **to_csv_kwargs):
    """

    @param df:
    @param method: mode is one of {{'infer', 'gzip', 'bz2', 'zip', 'xz', None}}
    @param archive_name:
    @return:
    """
    to_csv_kwargs['index'] = index
    to_csv_kwargs['compression'] = dict(method=method, archive_name=archive_name)
    df.to_csv(filename, **to_csv_kwargs)


# 绑定到pandas
dataframe_funcs = [
    to_compress,
    split
]
pd_bind(pd.DataFrame, *dataframe_funcs)

if __name__ == '__main__':
    # df = pd.DataFrame([[1, 2, 3] * 10000, [2, 2, 3] * 10000, [3, 2, 3] * 10000])
    #
    # import time
    #
    # s = time.time()
    # reduce_mem_usage(df)  # 34
    #
    # print(time.time() - s)
    from jinja2 import Template, Environment, PackageLoader, FileSystemLoader

    # env = Environment(loader=FileSystemLoader('./'))
    env = Environment(loader=PackageLoader('meutils'))
    template = env.get_template('df_html.j2')

    df = pd.DataFrame(range(10))
    content = template.render(df_to_html=df.to_html(), title="title", subtitle='subtitle')

    print(content)

    with open('./df.html', 'w') as fp:
        fp.write(content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : image_tools
# @Time         : 2024/8/28 13:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.apis.proxy.kdlapi import get_one_proxy

from meutils.decorators.retry import retrying, IgnoredRetryException

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.schemas.image_types import ImageRequest, ImagesResponse, HunyuanImageProcessRequest
from meutils.schemas.image_types import ImageProcess
from meutils.schemas.yuanbao_types import YUANBAO_BASE_URL, FEISHU_URL as YUANBAO_FEISHU_URL

from meutils.io.files_utils import to_bytes, to_base64, to_url_fal, to_url
from meutils.notice.feishu import send_message as _send_message

from fake_useragent import UserAgent

ua = UserAgent()

BASE_URL = "https://image.baidu.com"

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=jrWhAS"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@retrying(min=3, ignored_exception_types=(IgnoredRetryException,))
async def make_request_for_hunyuan(payload, token: Optional[str] = None, response_format: str = "url"):
    s = time.time()

    token = token or await get_next_token_for_polling(YUANBAO_FEISHU_URL)

    model = payload.pop("model", "removewatermark")

    logger.debug(payload)

    headers = {
        'cookie': token,
        'User-Agent': ua.random,
    }
    async with httpx.AsyncClient(base_url=YUANBAO_BASE_URL, headers=headers, timeout=100) as client:
        response = await client.post(f"/api/image/{model}", json=payload)
        response.raise_for_status()
        logger.debug(response.text)

        skip_strings = ['DONE', 'TRACEID']
        data = response.text.replace(r'\u0026', '&').splitlines() | xsse_parser(skip_strings=skip_strings)
        data = data and data[-1]
        logger.debug(data)

        # todo: 错误处理
        if isinstance(data, dict) and any(data["code"] == code for code in {"429"}):
            Exception(f"重试: {response.text}")

        elif isinstance(data, list) or any(i in response.text for i in {"当前图片没有检测到水印"}):  # 跳过重试并返回原始错误
            raise IgnoredRetryException(f"忽略重试: \n{response.text}")

        data = [
            {
                "url": data["imageUrl"],
                "imageUrl": data["imageUrl"],
                "thumbnailUrl": data["thumbnailUrl"],
            }
        ]
        if response_format == "url":
            return ImagesResponse(data=data, timings={"inference": time.time() - s})
        else:
            data[0]["b64_json"] = await to_base64(data[0]['url'])
            return ImagesResponse(data=data, timings={"inference": time.time() - s})


@retrying(min=3, ignored_exception_types=(IgnoredRetryException,))
async def make_request_for_gitee(payload, token: Optional[str] = None, response_format: str = "url"):
    s = time.time()
    feishu_url = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=gg5DNy"
    token = token or await get_next_token_for_polling(feishu_url)

    logger.debug(token)

    headers = {
        "Authorization": f"Bearer {token}",
        "X-Failover-Enabled": "true",
        "X-Package": "1910"
    }

    files = {
        "image": ("_.png", payload.pop('image'))
    }
    base_url = "https://ai.gitee.com/v1"
    async with httpx.AsyncClient(base_url=base_url, headers=headers, timeout=60) as client:
        response = await client.post("/images/mattings", data=payload, files=files)
        response.raise_for_status()
        response = ImagesResponse(**response.json())
        if response_format == "url":
            url = await to_url_fal(response.data[0].b64_json, content_type="image/png")
            response.data[0].url = url
            response.data[0].b64_json = None
            response.timings = {"inference": time.time() - s}

        return response


async def make_request_for_baidu(payload, token: Optional[str] = None, response_format: str = "url"):
    s = time.time()
    # token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True)
    headers = {
        # 'Cookie': token,
        'User-Agent': ua.random,
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
    }

    request_kwargs = {
        "proxy": await get_one_proxy(),
    }

    logger.debug(request_kwargs)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=120, **request_kwargs) as client:
        response = await client.post("/aigc/pccreate", data=payload)  # pcEditTaskid
        response.raise_for_status()
        data = response.json()

        logger.debug(data)

        image_base64 = None
        if task_id := data.get("pcEditTaskid"):
            for i in range(30):
                await asyncio.sleep(3)
                try:
                    response = await client.get(f'/aigc/pcquery?taskId={task_id}&', )  # todo: get任务未加代理
                    logger.debug(response.json())
                    if data := response.json().get("picArr", []):
                        image_base64 = data[0].get("src")
                        break
                except Exception as e:
                    logger.error(e)
                    if i > 3:
                        raise IgnoredRetryException(f"忽略重试: \n{response.text}")

        if not image_base64:
            raise Exception(f"NO WATERMARK FOUND: {data}")  #############

        if response_format == "url":
            url = await to_url(image_base64, content_type="image/png")

            return ImagesResponse(data=[{"url": url}], timings={"inference": time.time() - s})
        else:
            return ImagesResponse(data=[{"b64_json": image_base64}], timings={"inference": time.time() - s})


async def edit_image(request: ImageProcess):
    image, mask = await asyncio.gather(to_base64(request.image), to_base64(request.mask))

    # baidu_url = "https://chatfire.hkg.bcebos.com/zjz.jpg"
    # baidu_url = "https://cfcdn.bj.bcebos.com/zjz.jpg"
    # baidu_url =  "https://edit-upload-pic.cdn.bcebos.com/e9f47f610f22b5be48bd7dd45e1e5acd.jpeg?authorization=bce-auth-v1%2FALTAKh1mxHnNIyeO93hiasKJqq%2F2025-07-04T06%3A06%3A56Z%2F3600%2Fhost%2F7f46f4a362f50b39ec5cc2ff552a26a04b898cf66b71bdf8281150c0f7669c8e"

    payload = {
        "type": "1",  # 去水印

        "picInfo": image,
        "picInfo2": mask,

        # # 百度云盘 才会更快
        # "image_source": "1",
        # "original_url": baidu_url,
        # # # 更快但是会有错误
        # "thumb_url": baidu_url,
        # 更快但是会有错误

    }

    if request.model == "clarity":
        payload['type'] = "3"

        return await make_request_for_baidu(payload, response_format=request.response_format)
        # 临时替换
        # from meutils.apis.baidu.image_enhance import image_enhance
        # data = await image_enhance(request.image)
        # return ImagesResponse(data=[{"url": data.get("image")}])

    elif request.model == "remove-watermark":
        if mask:  ####### todo: mask 抠图
            payload['type'] = "2"
        return await make_request_for_baidu(payload, response_format=request.response_format)

    elif request.model == "clarity":
        payload['type'] = "3"

        return await make_request_for_baidu(payload, response_format=request.response_format)




    elif request.model == "expand":
        payload['type'] = "4"
        payload['ext_ratio'] = request.aspect_ratio
        return await make_request_for_baidu(payload, response_format=request.response_format)
    ################################################################################################

    elif request.model == "rmbg-2.0":
        payload = {
            "model": request.model,
            "image": await to_bytes(image),
        }
        return await make_request_for_gitee(payload, response_format=request.response_format)

    elif request.model.startswith("hunyuan-"):
        payload = {
            "imageUrl": request.image if request.image.startswith("http") else await to_url_fal(request.image),
        }
        # "remove-watermark" "clarity"
        if "remove-watermark" in request.model:
            payload["model"] = "removewatermark"
        elif "clarity" in request.model:
            payload["model"] = "clarity"

        return await make_request_for_hunyuan(payload, response_format=request.response_format)


if __name__ == '__main__':
    token = "BAIDUID=FF8BB4BF861992E2BF4A585A37366236:FG=1; BAIDUID_BFESS=FF8BB4BF861992E2BF4A585A37366236:FG=1; BIDUPSID=FF8BB4BF861992E2BF4A585A37366236; BDRCVFR[dG2JNJb_ajR]=mk3SLVN4HKm; userFrom=null; ab_sr=1.0.1_NjY5OWZiZDg5YTJmYTQzNWUyNzU1YjBmN2FlMDFiNjMyOTVhMDE3ZWVlYWY5N2Y2MTg4NGI1MzRmMmVjMjQyZjlhZTU2MmM1NDRlMmU4YzgwMzRiMjUyYTc4ZjY1OTcxZTE4OTA4YTlmMWIwZWUzNTdiMzlhZTRiM2IzYTQ0MjgyMzc2MjQwMGRlYzZlZDhjOTg5Yzg4NWVjMTNiZmVmZQ==; BDRCVFR[-pGxjrCMryR]=mk3SLVN4HKm; H_WISE_SIDS=60273_60360_60623_60664_60678_60684_60700"
    # hunyuan
    token = "web_uid=ac283ec7-4bf6-40c9-a0ce-5a2e0cd7db06; hy_source=web; hy_user=I09MgMfFcUUyVSIg; hy_token=hevVCi/QuVjQcre5NDRMO7FuiWCZoDMIq3Zp8IwNxrPUofl4zWYazHEdeZ2S5o7q; _qimei_q36=; _qimei_h38=f2d27f50f0f23e085296d28303000006a17a09; _qimei_fingerprint=efbb885a22f7d4e5589008c28bc8e7ba; _qimei_uuid42=18c0310102d1002a082420cd40bb9717523c3c7e12; _gcl_au=1.1.915258067.1733278380; _ga_RPMZTEBERQ=GS1.1.1733722091.3.1.1733722108.0.0.0; _ga=GA1.2.981511920.1725261466; sensorsdata2015jssdkcross=%7B%22distinct_id%22%3A%22100000458739%22%2C%22first_id%22%3A%22191b198c7b2d52-0fcca8d731cb9b8-18525637-2073600-191b198c7b31fd9%22%2C%22props%22%3A%7B%22%24latest_traffic_source_type%22%3A%22%E7%A4%BE%E4%BA%A4%E7%BD%91%E7%AB%99%E6%B5%81%E9%87%8F%22%2C%22%24latest_utm_medium%22%3A%22cpc%22%7D%2C%22identities%22%3A%22eyIkaWRlbnRpdHlfY29va2llX2lkIjoiMTkxYjE5OGM3YjJkNTItMGZjY2E4ZDczMWNiOWI4LTE4NTI1NjM3LTIwNzM2MDAtMTkxYjE5OGM3YjMxZmQ5IiwiJGlkZW50aXR5X2xvZ2luX2lkIjoiMTAwMDAwNDU4NzM5In0%3D%22%2C%22history_login_id%22%3A%7B%22name%22%3A%22%24identity_login_id%22%2C%22value%22%3A%22100000458739%22%7D%2C%22%24device_id%22%3A%22191b198c7b2d52-0fcca8d731cb9b8-18525637-2073600-191b198c7b31fd9%22%7D"

    # url = "https://api.chatfire.cn/beta/https://s3.ffire.cc/files/jimeng.jpg"
    # url = "https://juzhen-1318772386.cos.ap-guangzhou.myqcloud.com/mj/2025/06/07/7b347a36-8146-4d3d-a5dc-0b8dc365817d.png"
    url = "https://oss.ffire.cc/files/shuiyin.jpg"
    # url = "https://oss.ffire.cc/files/shuiyin3.jpg"

    # url = "https://s22-def.ap4r.com/bs2/upload-ylab-stunt-sgp/se/ai_portal_sgp_queue_mmu_txt2img_aiweb/9c520b80-efc2-4321-8f0e-f1d34d483ddd/1.png"

    request = ImageProcess(
        # model="hunyuan-remove-watermark",

        # model="remove-watermark",
        model="clarity",
        # model="expand",
        # model="rmbg-2.0",

        image=url,
        # mask=url,

        # response_format="b64_json"
    )
    arun(edit_image(request))

    # arun(image_edit(request))
    #
    # from urllib.parse import parse_qs, parse_qsl
    #
    # s = "query=bdaitpzs%E7%99%BE%E5%BA%A6AI%E5%9B%BE%E7%89%87%E5%8A%A9%E6%89%8Bbdaitpzs&picInfo=&picInfo2=&type=3&text=&ext_ratio=&expand_zoom=&original_url=https%3A%2F%2Fedit-upload-pic.cdn.bcebos.com%2F5a7311e6fc3425b307ac4359e4304431.jpeg%3Fauthorization%3Dbce-auth-v1%252FALTAKh1mxHnNIyeO93hiasKJqq%252F2025-07-02T10%253A42%253A07Z%252F3600%252Fhost%252Ff14fa759bffee374474540d0705a04c25068b40f432a720b4579056bac813a5f&thumb_url=https%3A%2F%2Fedit-upload-pic.cdn.bcebos.com%2F5a7311e6fc3425b307ac4359e4304431.jpeg%3Fauthorization%3Dbce-auth-v1%252FALTAKh1mxHnNIyeO93hiasKJqq%252F2025-07-02T10%253A42%253A07Z%252F3600%252Fhost%252Ff14fa759bffee374474540d0705a04c25068b40f432a720b4579056bac813a5f&front_display=0&create_level=0&image_source=1&style=&queryFeature=&imageFeature="
    # print(parse_qsl(s))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2023/11/22 15:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

import pysrt
from pysrt.srttime import SubRipTime

from pydub import AudioSegment
from pydub.playback import play


# 合并
def merge_audio_silent(audio_file, duration=1000, exclude=True):
    p = Path(audio_file)
    old_dir = p.parent.name
    new_dir = (p.parent.parent / f"{old_dir}_new")
    new_dir.mkdir(exist_ok=True)
    filename = new_dir / p.name

    audio = AudioSegment.from_file(audio_file)
    if exclude:
        duration -= len(audio)  # 剔除本身的长度
    audio += AudioSegment.silent(duration=max(duration, 0))
    audio.export(filename)
    return filename


def merge_audios(audio_files, filename: Optional[str] = None):
    audio = sum(map(AudioSegment.from_file, audio_files))
    filename = filename or "audio_merged.wav"
    audio.export(filename)
    return filename


def to_audio(file, filename="example.wav", format='wav', **kwargs):
    AudioSegment.from_file(file).export(filename, format=format, **kwargs)


if __name__ == '__main__':
    file = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/shaoyinceo.m4a"
    file = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/asr/audio-.mp3"
    file = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_audio/x1.wav"

    to_audio(file, filename="x1.mp3", format='mp3')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2024/10/21 20:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://platform.minimaxi.com/document/video_generation?key=66d1439376e52fcee2853049
# https://useapi.net/docs/start-here/setup-minimax
# token 过期时间一个月: 看下free hailuo
# https://jwt.io/

# todo: check token

import oss2

from meutils.pipe import *
from meutils.hash_utils import md5
from meutils.io.files_utils import to_bytes
from meutils.jwt_utils import decode_jwt_token
from meutils.schemas.hailuo_types import BASE_URL, FEISHU_URL
# from meutils.schemas.hailuo_types import BASE_URL_ABROAD as BASE_URL, FEISHU_URL_ABROAD as FEISHU_URL
from meutils.schemas.hailuo_types import BASE_URL_ABROAD

from meutils.schemas.hailuo_types import VideoRequest, VideoResponse
from meutils.llm.check_utils import check_tokens
from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message, VIDEOS
from meutils.apis.ppio import videos as ppio_videos

from meutils.apis.hailuoai.yy import get_yy

send_message = partial(
    _send_message,
    title=__name__,
    url=VIDEOS
)

APP_ID = '3001'
VERSION_CODE = '22202'

MODEL_MAPPING = {
    # video-01 video-01 video-01-live2d S2V-01

    "t2v-01": "23000",  # 23010
    "t2v-01-director": "23010",

    "i2v-01": "23001",
    "i2v-01-live": "23011",
    "video-01-live2d": "23011",
    "s2v-01": "23021",

    # "23210" # 要积分
}


def get_base_url(token):
    data = decode_jwt_token(token)
    if "小螺帽" not in str(data):
        logger.debug(data)

        return BASE_URL_ABROAD
    else:
        return BASE_URL


# minimax_video-01,minimax_video-01-live2d,,minimax_t2v-01,minimax_i2v-01,minimax_i2v-01-live,minimax_s2v-01


async def upload(file: bytes, token: str):
    data = await get_request_policy(token)

    access_key_id = data["accessKeyId"]
    access_key_secret = data["accessKeySecret"]
    security_token = data["securityToken"]
    bucket_name = data["bucketName"]
    endpoint = data["endpoint"]
    dir = data["dir"]

    # 创建OSS客户端    with timer():
    #         arun(hailuo_upload(file))
    auth = oss2.StsAuth(access_key_id, access_key_secret, security_token)
    bucket = oss2.Bucket(auth, endpoint, bucket_name)

    # file
    size = len(file)

    # 要上传的文件路径和文件名
    mimeType = "png"
    filename = f"{uuid.uuid4()}.{mimeType}"
    object_name = f"{dir}/{filename}"  # png写死
    bucket.put_object(object_name, file)

    params = {}
    payload = {
        "originFileName": "_.png",
        # "originFileName": "503af3b5-9c3b-4bdc-a6d4-256debce3dd5_00001_.png",

        "filename": filename,
        # "fileName": "db884a7c-99a4-40f5-929e-db8769dbf64a.png",

        "dir": dir,
        "endpoint": "oss-cn-wulanchabu.aliyuncs.com",
        "bucketName": "minimax-public-cdn",
        "size": f"{size}",
        "mimeType": mimeType,
        "fileMd5": md5(file),
        "fileScene": 10
    }
    headers = {
        'token': token
    }
    logger.debug(headers)

    BASE_URL = get_base_url(token)
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=120) as client:
        response = await client.post("/v1/api/files/policy_callback", params=params, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))
        return data

    # {'data': {'fileID': '307139766909288449',
    #           'ossPath': 'https://cdn.hailuoai.com/prod/2024-10-28-10/user/multi_chat_file/c9d8fc64-d9bf-42c0-8779-f39973579fca.png?image_process=resize,fw_320/format,webp'},
    #  'statusInfo': {'code': 0,
    #                 'debugInfo': '',
    #                 'httpCode': 0,
    #                 'message': '成功',
    #                 'requestID': '33a42a75-15e5-4d4e-b425-eeefc4d3f374',
    #                 'serverAlert': 0,
    #                 'serviceTime': 1730083971}}


@alru_cache(ttl=1 * 24 * 60 * 60)
@retrying()
async def get_access_token(token: str):
    BASE_URL = get_base_url(token)
    logger.debug(BASE_URL)

    logger.debug(f"get_access_token:{token}")

    params = {
        'device_platform': 'web',
        'app_id': APP_ID,
        'version_code': VERSION_CODE,
        'uuid': '8c059369-00bf-4777-a426-d9c9b7984ee6',
        'device_id': '243713252545986562',
        'os_name': 'Mac',
        'browser_name': 'chrome',
        'device_memory': '8',
        'cpu_core_num': '10',
        'browser_language': 'zh-CN',
        'browser_platform': 'MacIntel',
        'screen_width': '1920',
        'screen_height': '1080',
        'unix': f'{int(time.time())}000'
    }
    payload = {}
    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params=params, url="/v1/api/user/renewal")
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/v1/api/user/renewal", params=params, content=json.dumps(payload))
        response.raise_for_status()
        logger.debug(response.json())
        return response.json()['data']['token']


@alru_cache(ttl=1 * 60)
async def get_request_policy(token):
    """
    {
        "data": {
            "accessKeyId": "STS.NUVNbdQfqTNixCxUTAhdeJToq",
            "accessKeySecret": "DRBsh8Qm8VnXXxTwFMX5KkXVoqbbsPj4ewEfgTLysGvM",
            "securityToken": "CAISiwN1q6Ft5B2yfSjIr5bjBdjQvLlQ44yCemXJsVQUZOtJpZHEkzz2IHhMf3VpAusWsPw1n2tT6/sdlrBoS4JMREPJN5EhtsQLrl75PdKY4Jzltedb0EIf6JFQUUyV5tTbRsmkZj+0GJ70GUem+wZ3xbzlD2vAO3WuLZyOj7N+c90TRXPWRDFaBdBQVH0AzcgBLinpKOqKOBzniXayaU1zoVhYiHhj0a2l3tbmvHi4tlDhzfIPrIncO4Wta9IWXK1ySNCoxud7BOCjmCdb8EpN77wkzv4GqzLGuNCYCkkU+wiMN+vft9ZjKkg7RNBjSvMa8aWlzKYn57OCyt2v8XsXY7EJCRa4bZu73c7JFNmuMtsEbrvhMxzPqIvXbsKp6lh/MSxDblgRIId8dWURExUpTSrBIaOh6M4Bo5NbzHzuOsgSpnkVpz2AlbLiT9M/1aieRiRTcymwO/ayjeq6CeAF3mM8Mm0qPRouTM2+Zo5YD3N1opjTpiapdUYLox8awbuQLp25tMiF6FiLDvouuRqAAQJZHVcOeb5qnR6mkzw5hwzSOXoMXVFzDE2aB7dvRYFD9HiG6T66hE4Xlfpph9H7xWrpaBf5vqHQXp4gyuqVOgFIjdPECwisXlyAKQWMak7bGToh3cetCux3pjq74sP/KAjzSzDkwciJBbn8vZzNKKR/ozxqve925vtPPPWGPowwIAA=",
            "expiration": "2024-10-22T02:55:14Z",
            "dir": "cdn-yingshi-ai-com/prod/2024-10-22-09/user/multi_chat_file",
            "endpoint": "oss-cn-wulanchabu.aliyuncs.com",
            "bucketName": "minimax-public-cdn",
            "serverTime": "2024-10-22T02:10:07Z"
        }
    }
    """
    token = await get_access_token(token)

    headers = {
        'token': token,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
    }

    logger.debug(headers)

    BASE_URL = get_base_url(token)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get("/v1/api/files/request_policy")
        response.raise_for_status()
        return response.json()['data']
    # return {'accessKeyId': 'STS.NTjAXdazuDwvBugixueiNTwvW',
    #         'accessKeySecret': '4Uj1cuUQt2Xxnr21Yv41CUfGjnH8wBM3SB2Hn9sNrdDS',
    #         'bucketName': 'hailuo-video',
    #         'dir': 'moss/prod/2024-11-08-18/user/multi_chat_file',
    #         'endpoint': 'oss-us-east-1.aliyuncs.com',
    #         'expiration': '2024-11-08T11:38:31Z',
    #         'securityToken': 'CAISjwN1q6Ft5B2yfSjIr5ffCuLQjKVU87WdQFPWjXggaeZiu7LdtTz2IHhMe3lrB+kZsvo0lGFR5vcelrBoS4JMREPJNpMvv8UGq1LxMtKZ45PtseRZ0Fx+t9xLUUyV5tTbRsmkZsW/E67fejKpvyt3xqSAK1fGdle5MJqPpId6Z9AMJGeRZiZHA9EkUGkHr9QBZ0PMKdGnMBPhngi1C1Fz6C59kn9H4L6j2bfMiHzkgUb91/UeqqnoP5GgdLMMBppkVMqv1+EEAsiz2SVLuR9R7/U03u4W+T7GusyaRkdQ7xiBdKj2ioAzcFQhP/JmRfUe86Ghy6JC17aNx9Sn+XFkJvpIVinTfoekzfbfFfmhXtRDLu6jZCSWjInQaseu41J9MSNHLnNDY9tkMWNrTBs3UXTaI66j5VePfAavRq+VN3TYejPRcjQClPr9xDjnK93xuU5wsa8t6v7tUjA8KaFMoG7n3kGenWkdc5K/Z6udEt4wZ+3GM2OsuFNOQI8ZwZ+aRbfi3op7DyGXmSCG59Z1DPwagAF6Ybpx1FU5RRhp1G6yFZrwSz1uWkDL+5K5SYiq9g758m0ElogR5mgmkISgo2qy8/2br2XEA4eyRt5iIHSVX+lLrE2DIv4Cd65nZSQWX8G+av0D5qxCmspJEz3+VL4kdSxT42GuFpu9jhUtWLAX4K/5laoNyKR5LJHtZR3/pteBSCAA',
    #         'serverTime': '2024-11-08T10:38:31Z'}


# @retrying(predicate=lambda r: r.base_resp.status_code in {1000061, 1500009})  # 限流
async def create_task(request: VideoRequest, token: Optional[str] = None):
    # if request.model.lower() == "minimax-hailuo-02":  # 走派欧
    if 1:  # 全部走派欧
        request.model = "minimax-hailuo-02"
        _ = await ppio_videos.create_task(request)
        logger.debug(_)
        return _

    refresh_token = token or await get_next_token_for_polling(FEISHU_URL, from_redis=True, check_token=check_token)
    # refresh_token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDA0NzA4NzgsInVzZXIiOnsiaWQiOiIzMjg2MDg4ODkzNTA4MTU3NDQiLCJuYW1lIjoiUm9idXN0IEdlcnRydWRlIiwiYXZhdGFyIjoiaHR0cHM6Ly9jZG4uaGFpbHVvYWkudmlkZW8vbW9zcy9wcm9kLzIwMjQtMTItMjgtMTYvdXNlci91c2VyX2F2YXRhci8xNzM1Mzc1OTI0OTkyMTcxMDY3LWF2YXRhcl8zMjg2MDg4ODkzNTA4MTU3NDQiLCJkZXZpY2VJRCI6IjMwMjgzMzc1OTUxMjc2NDQxNyIsImlzQW5vbnltb3VzIjpmYWxzZX19.dLNBSHjqnKutGl3ralS2g8A-RodHjOdos11vdpbkPwc"

    BASE_URL = get_base_url(refresh_token)
    token = await get_access_token(refresh_token)

    payload = {
        "desc": request.prompt,
        "useOriginPrompt": not request.prompt_optimizer,
        "fileList": [],
        "modelID": "23000",  # 文生视频
        "quantity": f"{request.n}",

        # "extra": {
        #     "promptStruct": "{"value":[{"type":"paragraph","children":[{"text":"动起来"}]}],"length":3,"plainLength":3,"
        #     rawLength":3}",
        #     "templateID": ""
        # },

        # "durationType": 1,
        # "resolutionType": 3
    }

    if request.first_frame_image:
        file = await to_bytes(request.first_frame_image)
        if request.first_frame_image.startswith("http") and BASE_URL.endswith(".video") and 0:  # 必须上传
            file_data = {
                # "id": data['data']['fileID'],
                # "name": "_.png",
                # "type": "png",
                "url": request.first_frame_image,  #######
            }

            # {"desc": "跳起来", "useOriginPrompt": false, "fileList": [{"id": "338311163211288581",
            #                                                            "url": "https://cdn.hailuoai.video/moss/prod/2025-01-22-11/user/multi_chat_file/de5a4cec-eb26-4380-94e4-13b268bf5c0d.jpg",
            #                                                            "name": "duikoux.jpg", "type": "jpg"}],
            #  "modelID": "23021", "quantity": "1"}
        else:
            data = await upload(file, token=refresh_token)
            file_data = {
                "id": data['data']['fileID'],
                "name": "_.png",
                "type": "png",
                "url": data['data']['ossPath'],
            }

        payload["fileList"].append(file_data)
        payload["modelID"] = MODEL_MAPPING.get(request.model.lower(), "23001")

    logger.debug(bjson(payload))

    # payload = {
    #     "desc": "笑起来",
    #     "useOriginPrompt": True,
    #     "fileList": [
    #         {
    #             # "id": "335822260344578050",
    #             ############# 不需要走上传了
    #             # "url": "https://cdn.hailuoai.video/moss/prod/2025-01-15-13/user/multi_chat_file/6ca7b141-383c-4481-8bba-b9148c1339c0.jpg?x-oss-process=image/resize,p_50/format,webp",
    #             "url": "https://oss.ffire.cc/files/kling_watermark.png",
    #             "name": "baobao.jpg",
    #             "type": "jpg"
    #         }
    #     ],
    #     "modelID": "23001",
    #     "quantity": "1"
    # }

    params = {
        'device_platform': 'web',
        'app_id': APP_ID,
        'version_code': VERSION_CODE,
        'biz_id': 0,
        'uuid': '8c059369-00bf-4777-a426-d9c9b7984ee6',
        'device_id': '243713252545986562',
        'os_name': 'Mac',
        'browser_name': 'chrome',
        'device_memory': '8',
        'cpu_core_num': '10',
        'browser_language': 'zh-CN',
        'browser_platform': 'MacIntel',
        'screen_width': '1920',
        'screen_height': '1080',
        'unix': f'{int(time.time())}000'
    }

    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params),
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post(f"/api/multimodal/generate/video", params=params, content=json.dumps(payload))
        response.raise_for_status()

        data = response.json()

        logger.debug(bjson(data))

        task_id = (data.get('data') or {}).get('id', '')
        response = VideoResponse(
            task_id=f"hailuoai-{task_id}",
            base_resp=data.get('statusInfo', {}),
            system_fingerprint=refresh_token
        )
        if response.base_resp.status_code != 0:  # 451
            from fastapi import HTTPException, status
            raise HTTPException(status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
                                detail=response.base_resp)

        return response
    # {
    #     "statusInfo": {
    #         "code": 1000061,
    #         "httpCode": 0,
    #         "message": "上一个视频任务未完成，请稍后再试",
    #         "serviceTime": 1729512914,
    #         "requestID": "82bc8c60-4dc3-4ad0-b5b6-b1836e0c88ab",
    #         "debugInfo": "",
    #         "serverAlert": 0
    #     }
    # }

    # {
    #     "data": {
    #         "id": "304746220940677121"
    #     },
    #     "statusInfo": {
    #         "code": 0,
    #         "httpCode": 0,
    #         "message": "成功",
    #         "serviceTime": 1729513305,
    #         "requestID": "caaf2364-d2ed-45df-b79a-827810a5d58c",
    #         "debugInfo": "",
    #         "serverAlert": 0
    #     }
    # }


# 307134660730421250
async def get_task(task_id: str, token: str):
    if token.startswith("sk_"):
        return await ppio_videos.get_task(task_id)

    BASE_URL = get_base_url(token)

    logger.debug(BASE_URL)

    task_id = task_id.rsplit('-', 1)[-1]

    params = {
        'device_platform': 'web',
        'app_id': APP_ID,
        'version_code': VERSION_CODE,
        'biz_id': 0,
        'uuid': '8c059369-00bf-4777-a426-d9c9b7984ee6',
        'device_id': '243713252545986562',
        'os_name': 'Mac',
        'browser_name': 'chrome',
        'device_memory': '8',
        'cpu_core_num': '10',
        'browser_language': 'zh-CN',
        'browser_platform': 'MacIntel',
        'screen_width': '1920',
        'screen_height': '1080',
        'unix': f'{int(time.time())}000',
        'idList': task_id
    }

    payload = {}
    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params, url="/api/multimodal/video/processing"),
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get("/api/multimodal/video/processing", params=params)
        response.raise_for_status()
        data = response.json()

        logger.debug(bjson(data))

        response = VideoResponse(
            task_id=task_id,
            base_resp=data.get('statusInfo', {}),
            videos=(data.get('data') or {}).get("videos", [])[:1]
        )
        return response


# https://hailuoai.video/v1/api/user/equity?device_platform=web&app_id=3001&version_code=22201&uuid=3de88ad0-8a38-48a9-8ed3-0d63f9c71296&lang=en&device_id=302833759512764417&os_name=Mac&browser_name=chrome&device_memory=8&cpu_core_num=10&browser_language=zh-CN&browser_platform=MacIntel&screen_width=1920&screen_height=1080&unix=1731571578000
# @alru_cache(ttl=3600)
async def check_token(token, threshold: int = 30, **kwargs):
    BASE_URL = get_base_url(token)

    if not isinstance(token, str):
        return await check_tokens(token, check_token)

    try:
        token = await get_access_token(token)
        headers = {
            "token": token
        }
        params = {}
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
            response = await client.get("/v1/api/user/equity", params=params)
            response.raise_for_status()
            data = response.json()

            logger.debug(bjson(data))

            return any(i in str(data) for i in {"Ultra", "Unlimited", "高级会员"})
    except Exception as e:
        logger.error(e)
        logger.debug(token)
        return False


if __name__ == '__main__':  # 304752356930580482
    vip = True
    # vip = False
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzI5NzAyMjIsInVzZXIiOnsiaWQiOiIyNDM3MTMyNTI3OTc2NDA3MDgiLCJuYW1lIjoi5bCP6J665bi9NzA4IiwiYXZhdGFyIjoiaHR0cHM6Ly9jZG4ueWluZ3NoaS1haS5jb20vcHJvZC91c2VyX2F2YXRhci8xNzA2MjY3MzY0MTY0NDA0MDc3LTE3MzE5NDU3MDY2ODk2NTg5Nm92ZXJzaXplLnBuZyIsImRldmljZUlEIjoiMjQzNzEzMjUyNTQ1OTg2NTYyIiwiaXNBbm9ueW1vdXMiOnRydWV9fQ.X3KW00hAhSMk1c7DrXWYR27BROHNbfSiHD7Y-aweA6o"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM1NjMzNTIsInVzZXIiOnsiaWQiOiIyMjkwODQ3NTA2MDEzODgwMzciLCJuYW1lIjoi5bCP6J665bi9ODAzNyIsImF2YXRhciI6Imh0dHBzOi8vY2RuLnlpbmdzaGktYWkuY29tL3Byb2QvdXNlcl9hdmF0YXIvMTcwNjI2NzcxMTI4Mjc3MDg3Mi0xNzMxOTQ1NzA2Njg5NjU4OTZvdmVyc2l6ZS5wbmciLCJkZXZpY2VJRCI6IjMwNzIzNzc4MjU1NjE0NzcxMyIsImlzQW5vbnltb3VzIjpmYWxzZX19.MzZA9tW0YG2WFRWSkdD6bpEQt_0I-uIrxPJjxisKRNk"
    # 海外
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM4MDQwOTksInVzZXIiOnsiaWQiOiIzMDc5OTkyOTg5MDAwNzA0MDUiLCJuYW1lIjoiRGF2aWQgUGhpbGxpcHMiLCJhdmF0YXIiOiIiLCJkZXZpY2VJRCI6IiJ9fQ.wmmDC7XBmmlQvcPM5TpdGVQDQDIkMpF9nkYBSB7UBWw"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM1NjQ5NDYsInVzZXIiOnsiaWQiOiIzMDQ3NjQzOTU5NDEwOTMzNzYiLCJuYW1lIjoiTWFyaWEiLCJhdmF0YXIiOiIiLCJkZXZpY2VJRCI6IiJ9fQ.QYLSWMj85hB43blHcrfSNBYJ2v_gFmQO9DmIYC5sVuQ"

    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM1NDgxMzUsInVzZXIiOnsiaWQiOiIzMDI4MzM4Njc3NzE5NDkwNTgiLCJuYW1lIjoibWUgYmV0dGVyIiwiYXZhdGFyIjoiaHR0cHM6Ly9saDMuZ29vZ2xldXNlcmNvbnRlbnQuY29tL2EvQUNnOG9jSWdTU0NoczFENHNUajFTVGs3UHNUbTd5NTNKRFg5OW84QnhwWmNWNjU2MEFKYlJnPXM5Ni1jIiwiZGV2aWNlSUQiOiIifX0.b8lAlOd961nVrJunpW9tRAEETe6VwlEXvf7y2faNCeY"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM0MDY1MzUsInVzZXIiOnsiaWQiOiIzMDI4MzM4Njc3NzE5NDkwNTgiLCJuYW1lIjoibWUgYmV0dGVyIiwiYXZhdGFyIjoiaHR0cHM6Ly9saDMuZ29vZ2xldXNlcmNvbnRlbnQuY29tL2EvQUNnOG9jSWdTU0NoczFENHNUajFTVGs3UHNUbTd5NTNKRFg5OW84QnhwWmNWNjU2MEFKYlJnPXM5Ni1jIiwiZGV2aWNlSUQiOiIifX0.mcozMacSciz0MORdleOMS_uhrixhIlpQmFmUwvn81I4"
    # arun(get_access_token(token, vip=vip))
    # arun(get_request_policy(token, vip=vip))
    # arun(get_access_token(vip=False))

    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzM4MDQwOTksInVzZXIiOnsiaWQiOiIzMDc5OTkyOTg5MDAwNzA0MDUiLCJuYW1lIjoiRGF2aWQgUGhpbGxpcHMiLCJhdmF0YXIiOiIiLCJkZXZpY2VJRCI6IiJ9fQ.wmmDC7XBmmlQvcPM5TpdGVQDQDIkMpF9nkYBSB7UBWw"

    # p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/data/cowboy-hat-face.webp"
    # arun(upload(Path(p).read_bytes(), token=token, vip=vip))
    # arun(upload(Path(p).read_bytes(), vip=False))
    # access_token = arun(get_access_token())
    # arun(upload(Path("img.png").read_bytes(), token=access_token))
    # arun(upload(Path("img.png").read_bytes(), token=access_token))

    # arun(get_task(task_id="307137575113703427", token=token)) # 307173162217783304
    # arun(get_task(task_id="307148849188945924", token=token))

    # arun(get_task(task_id="307267574751313927", token=token))

    # arun(get_task(task_id="307177115102699528", token=token))

    token = None
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTM2ODcxNjksInVzZXIiOnsiaWQiOiIzOTEzNTgxNTc5OTY5MjA4MzUiLCJuYW1lIjoienBiZSB4bnp3IiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMDI4MzM3NTk1MTI3NjQ0MTciLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.v6IcCd74UiMLepoqq_fpSTbABUAxXnb9seyCk9SAOec"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDQ3MDMwNzIsInVzZXIiOnsiaWQiOiIzMDI4MzM4Njc3NzE5NDkwNTgiLCJuYW1lIjoibWUgYmV0dGVyIiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMDI4MzM3NTk1MTI3NjQ0MTciLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.Mjb64ZjkKyV9pj-_bXyLczU6kU729VLaKbYj9NmrK-4"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDg3Mzg4MTQsInVzZXIiOnsiaWQiOiIyMjkwODQ3NTA2MDEzODgwMzciLCJuYW1lIjoi5bCP6J665bi9ODAzNyIsImF2YXRhciI6Imh0dHBzOi8vY2RuLmhhaWx1b2FpLmNvbS9wcm9kL3VzZXJfYXZhdGFyLzE3MDYyNjc3MTEyODI3NzA4NzItMTczMTk0NTcwNjY4OTY1ODk2b3ZlcnNpemUucG5nIiwiZGV2aWNlSUQiOiIyNDM3MTMyNTI1NDU5ODY1NjIiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.o0SoZMSTWkXNHxJjt3Ggby5MJWSfd-rnK_I95T_WMP8"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTAxMjg4NTIsInVzZXIiOnsiaWQiOiIzNzQwMTM3NzUyNzg4ODY5MTciLCJuYW1lIjoiTmFodWVsIE1vbGluYSIsImF2YXRhciI6IiIsImRldmljZUlEIjoiMzEzMzc0MTIyMjEyMjc4MjczIiwiaXNBbm9ueW1vdXMiOmZhbHNlfX0.uxTtDTcPT07piVA-x3N2ms2VrRN3JwcU99g_HJLwqLE"
    request = VideoRequest(
        # model="t2v-01",
        model="I2V-01-live",
        # model="S2V-01-live",

        # prompt="smile",  # 307145017365086216
        prompt="动起来",  # 307145017365086216
        first_frame_image="https://oss.ffire.cc/files/kling_watermark.png"  # 307173162217783304
    )

    r = arun(create_task(request, token=token))

    # arun(get_task(task_id=r.task_id, token=r.system_fingerprint))

    # arun(get_task(task_id="hailuoai-378260932722450439", token=token))

    # arun(get_access_token(token))
    #
    #
    # data = {
    #     "model": "video-01",
    #     "prompt": "画面中两个人非常缓慢地拥抱在一起",
    #     "prompt_optimizer": True,
    #     # "first_frame_image": "https://hg-face-domestic-hz.oss-cn-hangzhou.aliyuncs.com/avatarapp/ai-cache/54883340-954c-11ef-8920-db8e7bfa3fdf.jpeg"
    # }
    # request = VideoRequest(**data)
    # r = arun(create_task(request, vip=vip))
    # arun(get_task(task_id=r.task_id, token=r.system_fingerprint))

    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3Mzg0Njk5MTUsInVzZXIiOnsiaWQiOiIzMjY0MzI4MjI3OTYxMzY0NTYiLCJuYW1lIjoiRGVib3JhaCBNaWxsZXIiLCJhdmF0YXIiOiIiLCJkZXZpY2VJRCI6IjMxMzMxNTk1MzEwMDQxMDg4NyIsImlzQW5vbnltb3VzIjpmYWxzZX19.ZnpAgRPwtc4JZ2B0PbEfgMU_I4_YRtifzNRVHa5g90U"

    # arun(get_task("hailuoai-307495165395488768", token=token))
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzUwMTc3ODAsInVzZXIiOnsiaWQiOiIzMTEyOTUzMTkzMjc1NzYwNjQiLCJuYW1lIjoiVUdIUiBKVkJYIiwiYXZhdGFyIjoiaHR0cHM6Ly9saDMuZ29vZ2xldXNlcmNvbnRlbnQuY29tL2EvQUNnOG9jS3RuR2NjdGZsWV9fR2tiQ1MzdnhzSXdWSEFUX0ZmMFdyb3RvMnN4bFdWZW1KMm53PXM5Ni1jIiwiZGV2aWNlSUQiOiIzMTMzMTU5NTMxMDA0MTA4ODciLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.cyZifq4FQl46P5_acTNT04qu2GVDDeSBbwjw3J1vWPo"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3MzUwMzQ1MjcsInVzZXIiOnsiaWQiOiIzMTMzODk5MjA0NjA5MzkyNjgiLCJuYW1lIjoiY2l4ZiB4YmNnIiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMTM0MDgyMjg0NTEwOTg2MjYiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.eOtAUe3MmarOGNk64j0bfaLNBZ4yxkqwIi1tUhOFD5c"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTAxMjg4NTIsInVzZXIiOnsiaWQiOiIzNzQwMTM3NzUyNzg4ODY5MTciLCJuYW1lIjoiTmFodWVsIE1vbGluYSIsImF2YXRhciI6IiIsImRldmljZUlEIjoiMzEzMzc0MTIyMjEyMjc4MjczIiwiaXNBbm9ueW1vdXMiOmZhbHNlfX0.uxTtDTcPT07piVA-x3N2ms2VrRN3JwcU99g_HJLwqLE"
    # token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTAxMjg4NTIsInVzZXIiOnsiaWQiOiIzNzQwMTM3NzUyNzg4ODY5MTciLCJuYW1lIjoiTmFodWVsIE1vbGluYSIsImF2YXRhciI6IiIsImRldmljZUlEIjoiMzEzMzc0MTIyMjEyMjc4MjczIiwiaXNBbm9ueW1vdXMiOmZhbHNlfX0.uxTtDTcPT07piVA-x3N2ms2VrRN3JwcU99g_HJLwqLE"
    # token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTMxNzMyMjgsInVzZXIiOnsiaWQiOiIzODkxMjkzNTAyMjk0MTM4OTIiLCJuYW1lIjoibG53ciBqb2R1IiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMTMzNzQxMjIyMTIyNzgyNzMiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.XIScaKmCg2OfBGoB-QD-J9Q1SEJuL41l5SqOFpHVWbc"
    # token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NTMxNzMyMjgsInVzZXIiOnsiaWQiOiIzODkxMjkzNTAyMjk0MTM4OTIiLCJuYW1lIjoibG53ciBqb2R1IiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMTMzNzQxMjIyMTIyNzgyNzMiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.XIScaKmCg2OfBGoB-QD-J9Q1SEJuL41l5SqOFpHVWbc"
    # arun(check_token(token))

# httpx.HTTPStatusError: Client error '403 Forbidden' for url 'https://hailuoai.video/api/multimodal/generate/video?device_platform=web&app_id=3001&version_code=22202&biz_id=0&uuid=8c059369-00bf-4777-a426-d9c9b7984ee6&device_id=243713252545986562&os_name=Mac&browser_name=chrome&device_memory=8&cpu_core_num=10&browser_language=zh-CN&browser_platform=MacIntel&screen_width=1920&screen_height=1080&unix=1751036910000'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : cache_utils
# @Time         : 2021/11/24 上午11:09
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : https://cachetools.readthedocs.io/en/stable/

# https://github.com/awolverp/cachebox 缓存 todo
"""
异步缓存 from aiocache import cached

FIFO：First In、First Out，就是先进先出。

LFU：Least Frequently Used，就是淘汰最不常用的。

LRU：Least Recently Used，就是淘汰最久不用的。

MRU：Most Recently Used，与 LRU 相反，淘汰最近用的。

RR：Random Replacement，就是随机替换。

TTL：time-to-live 的简称，也就是说，Cache 中的每个元素都是有过期时间的，如果超过了这个时间，那这个元素就会被自动销毁。
如果都没过期并且 Cache 已经满了的话，那就会采用 LRU 置换算法来替换掉最久不用的，以此来保证数量。
"""
import time
import sys
import math
import pickle
import hashlib
import numpy as np
import pandas as pd
import inspect
import joblib

from typing import Iterable, Optional
from functools import lru_cache as _lru_cache
from pathlib import Path
from joblib import Memory
from joblib.func_inspect import filter_args
from joblib import hashing  # hashing.hash

from loguru import logger
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor

from tqdm.auto import tqdm
from cachetools import cached, cachedmethod, LRUCache, RRCache, TTLCache as _TTLCache, keys

# ME
from meutils.decorators import decorator, singleton, background_task
from meutils.hash_utils import md5

TTLCache = _lru_cache()(_TTLCache)


# todo: cache_clear 很重要
def map_cache():
    return cached({})


# hash只支持单入参
key_fn = lambda *args, **kwargs: hashing.hash((args, kwargs))


def ttl_cache(ttl=np.inf, maxsize=1024, key=key_fn):
    """https://cachetools.readthedocs.io/en/stable/

        @ttl_cache()
        @disk_cache()
        def fn(x):  # 多级缓存
            time.sleep(1)
            return x

    @param ttl:
    @param maxsize:
    @param key: key_fn = lambda *args, **kwargs: f"{args}{kwargs}"
    @return:
    """
    # hash = lambda *args, **kwargs: hashing.hash()

    return cached(TTLCache(maxsize, ttl), key=key)  # LRUCache


@decorator
def redis_cache(func, rc=None, ex=3, ignore=None, verbose=1, *args, **kwargs):
    """redis 缓存"""
    import redis
    rc = rc or redis.Redis()

    ##############################################################################
    fn_name = func.__name__ if hasattr(func, '__name__') else 'fn'

    raw_key = dict(filter_args(func, [], list(args), kwargs))
    raw_key = {**raw_key.pop('**', {}), **raw_key}

    # self cls 看场景选择
    for arg in ignore or []:
        raw_key.pop(arg, None)

    key = hashing.hash(raw_key)
    k = f"cache:{fn_name}:{key}"

    ##############################################################################

    if k in rc:
        return pickle.loads(rc.get(k))
    else:
        if verbose: logger.debug(f"CacheKey: {k}")

        _ = func(*args, **kwargs)
        rc.set(k, pickle.dumps(_), ex=ex)
        return _


@decorator
def joblib_cache(func, location='cachedir', ignore=None, verbose=1, *args, **kwargs):  # todo
    """硬盘缓存

    @param func:
    @param location:
    @param ignore: 有时我们不希望因某些参数的改变而导致重新计算，例如调试标志。
        @memory.cache(ignore=['debug'])
        def my_func(x, debug=True):
            print('Called with x = %s' % x)
    @param verbose:
    @param args:
    @param kwargs:
    @return:
    """
    memory = Memory(location=location, verbose=verbose)
    return memory.cache(func, ignore)(*args, **kwargs)


@decorator
def diskcache(func, location='cachedir', ttl=None, ignore: Optional[list] = None, verbose=1, tag=None, *args, **kwargs):
    """
    https://zhuanlan.zhihu.com/p/356447502
    https://grantjenks.com/docs/diskcache/

    """
    from diskcache import Cache, FanoutCache

    cache = FanoutCache(directory=location)

    ##############################################################################
    raw_key = dict(filter_args(func, [], list(args), kwargs))
    raw_key = {**raw_key.pop('**', {}), **raw_key, '__tag__': tag or func.__name__}

    # self cls 看场景选择
    ignore = ignore or []
    # ignore += [
    #     'api_base', 'api_key', 'openai_api_base', 'openai_api_key',
    #     'organization', 'openai_organization',
    #     'request_timeout'
    # ]
    for arg in ignore:
        raw_key.pop(arg, None)

    key = hashing.hash(raw_key)

    ##############################################################################

    _ = cache.get(key, '__NO__', retry=True)
    if isinstance(_, str) and _ == '__NO__':
        _ = func(*args, **kwargs)

        cache.set(key, _, expire=ttl)  # 异步写入

        if verbose:
            from loguru import logger
            logger = logger.patch(lambda r: r.update(name=func.__name__))
            logger.debug(f"{cache.directory}: `CacheKey: {raw_key if verbose != 1 else key}`")

    return _


@decorator
def disk_cache(func, location='cachedir', maxsize=128, ttl=np.inf, verbose=True, *args, **kwargs):
    ttl_cache = TTLCache(maxsize, ttl)  # 单例

    k = md5(f"cache_{func.__name__}_{args}_{kwargs}")  # uuid
    output = Path(location) / Path(k) / '__output.pkl'

    if (ttl == np.inf or k in ttl_cache) and output.is_file():  # ttl=np.inf 不作key判断, 相当于maxsize无穷大
        # return joblib.load(output)
        return pickle.load(open(output, 'rb'))

    else:

        _ = func(*args, **kwargs)

        # @background_task
        def dump():
            if verbose: logger.info(f"CacheKey: {k}")

            output.parent.mkdir(parents=True, exist_ok=True)
            pickle.dump(_, open(output, 'wb'))
            ttl_cache[k] = 0  # 更新cache

        dump()
        return _


class MomentoCache(object):

    def __init__(self, cache_name='TEST', ttl=24 * 3600):
        from datetime import timedelta

        from momento import CacheClient, Configurations, CredentialProvider

        self.cache_name = cache_name
        self.cache_client = CacheClient(
            configuration=Configurations.Laptop.v1(),
            credential_provider=CredentialProvider.from_environment_variable('MOMENTO_AUTH_TOKEN'),
            default_ttl=timedelta(seconds=ttl)
        )
        self.cache_client.create_cache(self.cache_name)

    def set(self, key, value, expire=None):
        try:
            self.cache_client.set(self.cache_name, key, pickle.dumps(value))
            return True
        except Exception as e:
            logger.error(e)
            return False

    def get(self, key, default=None):
        from momento.responses import CacheGet, CacheSet, CreateCache

        get_response = self.cache_client.get(self.cache_name, key)
        if isinstance(get_response, CacheGet.Hit):
            return pickle.loads(get_response.value_bytes)
        return default


@decorator
def cache(func, cache_name='cachedir', ttl=None, ignore=None, verbose=1, tag=None, *args, **kwargs):
    cache = MomentoCache(cache_name)  # 缓存类

    raw_key = dict(filter_args(func, [], list(args), kwargs))

    ignore = (ignore or []) + ['self', 'cls', 'api_base', 'api_key', 'organization', 'request_timeout']  # self 本身就不被计入
    raw_key = {**raw_key.pop('**', {}), **raw_key, '__tag__': tag or func.__name__}
    for arg in ignore:
        raw_key.pop(arg, None)

    key = hashing.hash(raw_key)

    _ = cache.get(key)
    if _ is None:
        _ = func(*args, **kwargs)
        if inspect.isgenerator(_):
            _ = list(_)
            cache.set(f"{key}_isgenerator", True, expire=ttl)

        cache.set(key, _, expire=ttl)  # 异步写入

        if verbose:
            from loguru import logger
            logger = logger.patch(lambda r: r.update(name=func.__name__))
            logger.info(f"{cache.cache_name}: `CacheKey: {raw_key if verbose != 1 else key}`")

    return _ if not cache.get(f"{key}_isgenerator") else iter(_)


if __name__ == '__main__':
    # import time
    #
    #
    # @redis_cache(ex=15 * 60)
    # def f(x):
    #     time.sleep(1)
    #     return x
    #
    #
    # for i in range(110):
    #     print(f(i))
    #     print(f(i))

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/5/23 13:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *



import requests
import json

url = "https://tools.dreamfaceapp.com/dw-server/face/animate_image_web"

payload = json.dumps({
   "aigc_img_no_save_flag": False,
   "template_id": "6606889f54e4e700070db4b1",
   "app_version": "4.7.1",
   "timestamp": 1747978119668,
   "user_id": "d7668b9337aa943709a4387101583ab1",
   "no_water_mark": 0,
   "merge_by_server": False,
   "account_id": "67fcdea4ae936300076d8d89",
   "pt_infos": [
      {
         "lan": "en",
         "audio_id": "6d770a8819e946a797aec5bb43547142",
         "context": "a dog",
         "voice_engine_id": "prompt_97f7a53122124635be47b3a773c061e6",
         "video_url": "https://cdns3.dreamfaceapp.com/web/common/material/219e971f53e54f3bb8fe9d55e12a1688.mp4"
      }
   ],
   "work_type": "AVATAR_VIDEO",
   "santa_info": {
      "email": "",
      "signature": ""
   },
   "photo_info_list": [
      {
         "photo_path": "",
         "origin_face_locations": [
            {
               "left_upper_x": 0,
               "left_upper_y": 0,
               "right_width": 1,
               "down_high": 1
            }
         ],
         "square_face_locations": [
            {
               "left_upper_x": 0,
               "left_upper_y": 0,
               "down_high": 1,
               "right_width": 1
            }
         ],
         "five_lands": [
            [
               [
                  1,
                  1
               ],
               [
                  1,
                  1
               ],
               [
                  1,
                  1
               ],
               [
                  1,
                  1
               ],
               [
                  1,
                  1
               ]
            ]
         ],
         "face_nums": 1,
         "mask_path": ""
      }
   ],
   "play_types": [
      "VIDEO",
      "PT"
   ],
   "ext": {
      "track_info": "{}",
      "sing_title": "a dog",
      "animate_channel": "dynamic"
   }
})
headers = {
   'dream-face-web': 'dream-face-web',
   'priority': 'u=1, i',
   'token': 'eyJhbGciOiJIUzI1NiJ9.eyJleHAiOjE3NDQ2NTQxMTYsInBheWxvYWQiOnsiaWQiOiI2N2ZjZGVhNGFlOTM2MzAwMDc2ZDhkODkiLCJ0aGlyZFBsYXRmb3JtIjoiR09PR0xFIiwidGhpcmRJZCI6IjExMTQ5Nzk1NTUyNjQxMTE4MjUwNiIsInBhc3N3b3JkIjpudWxsLCJ0aGlyZEV4dCI6eyJlbWFpbCI6ImFpY2hhdGZpcmVAZ21haWwuY29tIiwibmFtZSI6Im1lIGJldHRlciIsImdlbmRlciI6bnVsbCwiYmlydGhkYXkiOm51bGwsIm5pY2tOYW1lIjpudWxsLCJnaXZlbk5hbWUiOiJtZSIsImZhbWlseU5hbWUiOiJiZXR0ZXIiLCJwcm9maWxlUGljdHVyZSI6Imh0dHBzOi8vbGgzLmdvb2dsZXVzZXJjb250ZW50LmNvbS9hL0FDZzhvY0lnU1NDaHMxRDRzVGoxU1RrN1BzVG03eTUzSkRYOTlvOEJ4cFpjVjY1NjBBSmJSZz1zOTYtYyJ9LCJjcmVhdGVUaW1lIjoxNzQ0NjI1MzE2MDIyLCJ1cGRhdGVUaW1lRm9ybWF0IjoiMjAyNS0wNC0xNCAxODowODozNi4wMjIiLCJkZWxldGUiOm51bGwsImNyZWF0ZVRpbWVGb3JtYXQiOiIyMDI1LTA0LTE0IDE4OjA4OjM2LjAyMiIsInVwZGF0ZVRpbWUiOjE3NDQ2MjUzMTYwMjIsInBsYXRmb3JtVHlwZSI6Mn19.Rxt41B__ExhAHdmCHMiDDOkFl8IddSena5lsxbnNEX0',
   'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
   'content-type': 'application/json'
}

response = requests.request("POST", url, headers=headers, data=payload)

print(response.text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : reasoner
# @Time         : 2025/2/6 08:35
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
1. 适配任意客户端 think标识输出
2. 开源标准化
todo: 对齐官网格式
"""

from openai import AsyncOpenAI

from meutils.pipe import *
from meutils.llm.clients import chatfire_client, AsyncOpenAI
from meutils.llm.openai_utils import to_openai_params

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk, ChatCompletionRequest, CompletionUsage

reasoner_system = {
    'role': 'system',
    'content': '你是个AI助手，请务必在深度思考之后，再回答用户问题'
}
# re.DOTALL 标志使得 . 可以匹配换行符
think_pattern = re.compile(r'<think>(.*?)</think>', re.DOTALL)


def extract_think_content(content):
    # 匹配<think>和</think>之间的内容，包括换行符
    # re.DOTALL 标志使得 . 可以匹配换行符
    think_content = think_pattern.search(content)

    if think_content:
        # 获取think标签中的内容
        think_text = think_content.group(1)
        # 移除整个think标签及其内容
        cleaned_content = think_pattern.sub('', content)
        # 返回think中的内容和清理后的文本
        return think_text.strip(), cleaned_content.strip()
    return "", content


class Completions(object):

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, reasoning_stream: bool = True):

        self.reasoning_stream = reasoning_stream

        self.client = AsyncOpenAI(
            base_url=base_url, api_key=api_key,
        )

    async def create(self, request: ChatCompletionRequest):
        """适配任意客户端"""
        s = time.perf_counter()
        request.messages.insert(0, reasoner_system)

        data = to_openai_params(request)

        if request.stream:
            completion_chunks = await self.client.chat.completions.create(**data)

            is_reasoning_content = True
            # reasoning_prefix = "> Reasoning\n"
            # reasoning_suffix = "Reasoned for "
            reasoning_prefix = "```reasoning\n"
            reasoning_suffix = "\n```\n"

            async for chunk in completion_chunks:
                # chunk.model = "deepseek-reasoner"
                message = chunk.choices[0].delta
                message.content = message.content or ""  ########## 损失了？message chunk #########应该是很多值未带进chunk 中转站自定义渠道bug 参数放请求头吧

                # logger.debug(message)
                if not hasattr(message, 'reasoning_content'):  # 标准化  reasoning_content
                    message.reasoning_content = ""
                    if is_reasoning_content:
                        message.reasoning_content = (
                            message.content.replace("<think>", "")
                            .replace("</think>", "")
                            .replace("\n\n", "\n")
                        )
                        if "</think>" in message.content:  # 思考结束
                            is_reasoning_content = False
                        message.content = ""

                    # logger.debug(message)

                if self.reasoning_stream:  # 适配任意客户端: 展示推理内容

                    if message.reasoning_content and message.reasoning_content.strip():  # 思考开始
                        message.content = f"{reasoning_prefix}{message.reasoning_content}"
                        reasoning_prefix = ""

                    elif message.content:  # 思考结束
                        if reasoning_suffix:
                            message.content = f"{message.content.rstrip()}{reasoning_suffix}"
                            # message.content = f"{reasoning_suffix} {time.perf_counter() - s:.0f} seconds.\n{message.content}"
                            reasoning_suffix = ""

                    yield chunk
                    # logger.debug(chunk)
                else:
                    yield chunk
                    # logger.debug(chunk)


        else:  # 非流
            completions = await self.client.chat.completions.create(**data)

            completions.model = "deepseek-reasoner"
            message = completions.choices[0].message

            if not hasattr(message, 'reasoning_content'):
                reasoning_content, content = extract_think_content(message.content)

                if content.strip().startswith("<think>"):  # max tokens
                    reasoning_content = content.replace("<think>", "")
                    content = ""

                completions.choices[0].message.reasoning_content = reasoning_content
                completions.choices[0].message.content = content

            yield completions


if __name__ == '__main__':
    # [
    #     "qwen-plus-latest",
    #     "qvq-72b-preview",
    #     "qwq-32b-preview",
    #     "qwen2.5-coder-32b-instruct",
    #     "qwen-vl-max-latest",
    #     "qwen-turbo-latest",
    #     "qwen2.5-72b-instruct",
    #     "qwen2.5-32b-instruct"
    # ]
    request = ChatCompletionRequest(
        # model="qwen-turbo-2024-11-01",
        # model="qwen-max-latest",
        # model="qwen-plus-latest",

        # model="deepseek-r1:1.5b",
        model="DeepSeek-R1-Distill-Qwen-1.5B",
        # model="deepseek-r1:32b",
        #
        # model="deepseek-r1",
        # model

        messages=[
            # {
            #     'role': 'system',
            #     'content': '深度思考之后在回答问题并给出详细的理由'
            # },
            {
                'role': 'system',
                'content': '你是个AI助手，请务必在深度思考之后，再回答用户问题'
            },
            {
                'role': 'user',
                'content': '你好'
            },

        ],
        stream=True,
    )
    arun(Completions(
        base_url="https://ai.gitee.com/v1",
        api_key="AHKZ65ARNFH8QGMUUVCPECDTZVOLRPUXIKPGAC1Y",
    ).create(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : image_tools
# @Time         : 2024/8/28 13:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_spreadsheet_values, get_next_token_for_polling
from meutils.schemas.openai_types import ImageRequest, ImagesResponse
from meutils.schemas.image_types import BAIDU_TASKS, BaiduImageProcessRequest

from meutils.decorators.retry import retrying
from meutils.schemas.image_types import ASPECT_RATIOS
from meutils.oss.minio_oss import Minio
from meutils.decorators.contextmanagers import try_catcher
from meutils.schemas.baidu_types import BDAITPZSRequest
from meutils.schemas.task_types import Task, TaskType
from meutils.io.image import image_to_base64, crop_polygon
from meutils.io.files_utils import to_bytes, to_base64, to_url
from meutils.apis.proxy.kdlapi import get_one_proxy
from fake_useragent import UserAgent

ua = UserAgent()
from meutils.notice.feishu import send_message as _send_message

BASE_URL = "https://image.baidu.com"

url = "https://image.baidu.com/aigc/pccreate"

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=jrWhAS"

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@retrying(max_retries=3, title=__name__)
async def create_task(
        request: BDAITPZSRequest,
        token: Optional[str] = None,
        is_async: bool = True,
        response_format: Optional[Literal["url", "b64_json"]] = None
):
    token = token or await get_next_token_for_polling(feishu_url=FEISHU_URL, from_redis=True)

    request.picInfo = await to_base64(request.original_url)

    payload = request.model_dump()

    headers = {
        # 'X-Requested-With': 'XMLHttpRequest',
        'Cookie': token,
        'User-Agent': ua.random,
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
    }

    request_kwargs = {
        "proxy": await get_one_proxy()
    }

    logger.debug(request_kwargs)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100, **request_kwargs) as client:

        response = await client.post('/aigc/pccreate', data=payload)

        logger.debug(response.status_code)
        # logger.debug(response.text)
        # {"status":5,"message":"网络异常，请稍候再试~","isGenerate":true} 服务端url访问不了
        # {'status': 0, 'resType': 2, 'timestamp': '1727233935', 'token': '5a5fc23b052b31d2120abab9c1d02178'} 没有水印

        if response.is_success:
            data = response.json()

            if "pcEditTaskid" not in data:  # data.get('status') == 0
                send_message(f"照片失败\n{request.original_url}\n{request.model_dump_json(indent=4)}")
                raise Exception(f"无法处理该照片，可联系管理员\n{data}")

            if is_async:
                # {
                #     "status": 0,
                #     "pcEditTaskid": "cr7sbuue1pnfd61bjgu0",
                #     "resType": 0,
                #     "timestamp": "1724892667",
                #     "token": "e668cb4457494351f65300d9f388bb2b"
                # }

                task_id = f"{TaskType.pcedit}-{data['pcEditTaskid']}"
                return Task(id=task_id, data=data, system_fingerprint=token)

            else:  # {'algoprocess': 0, 'isGenerate': False, 'progress': 4}
                logger.debug(data)
                task_id = data['pcEditTaskid']

                for i in range(1, 30):
                    await asyncio.sleep(2 / i)
                    with try_catcher():
                        data = await get_task(task_id, response_format=response_format)
                        logger.debug(f"progress: {data['progress']}")
                        if data['isGenerate']:
                            return data

        response.raise_for_status()


@retrying(max_retries=3, title=__name__)
async def get_task(
        task_id,
        token: Optional[str] = None,
        response_format: Optional[Literal["url", "b64_json"]] = None
):
    task_id = task_id.split("-", 1)[-1]

    params = {
        "taskId": task_id
    }
    request_kwargs = {
        "proxy": await get_one_proxy()
    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=30, **request_kwargs) as client:
        response = await client.get('/aigc/pcquery', params=params)
        data = response.json()

        if response_format == "url":
            for arr in data.get('picArr', []):
                if base64_string := arr.pop('src', None):
                    data['url'] = await to_url(base64_string, filename=f"{task_id}.png")

        return data


TASKS = {
    "matting": '9',
    "removewatermark": '1'
}


async def image_process(request: BaiduImageProcessRequest):
    _request = BDAITPZSRequest(type=TASKS.get(request.task), original_url=request.image, thumb_url=request.image)
    data = await create_task(_request, is_async=False, response_format="url")

    if request.task == "matting":
        outline_points = []
        inline_points = []

        if "foreground" not in data:  # 未识别到主体： todo: mask
            return {"data": [{"url": data['url']}]}

        for outlines in data['foreground'][0].get('outlines', []):
            _ = list(map(lambda point: (point['x'], point['y']), outlines['line']))
            outline_points.append(_)

        for inlines in data['foreground'][0].get('inlines', []):
            _ = list(map(lambda point: (point['x'], point['y']), inlines['line']))
            inline_points.append(_)

        image = await to_bytes(request.image)
        result = crop_polygon(image, outline_points, inline_points)

        # 将 PIL Image 转换为字节
        buffer = io.BytesIO()
        result.save(buffer, format="PNG")
        byte_data = buffer.getvalue()
        url = await to_url(byte_data)

        return {"data": [{"url": url}]}
    return {"data": [{"url": data['url']}]}


if __name__ == '__main__':
    url = "https://oss.ffire.cc/files/kling_watermark.png"
    # url = "https://env-00jxgna201cb.normal.cloudstatic.cn/ori/tmp_dc12fc648ab10c4b8d310f3e8645781278e556a0264836d7fdb806c6bb83c493.jpeg"

    # 涂抹消除
    # url = "https://env-00jxgna201cb.normal.cloudstatic.cn/ori/tmp_dc12fc648ab10c4b8d310f3e8645781278e556a0264836d7fdb806c6bb83c493.jpeg"
    # url_ = "https://env-00jxgna201cb.normal.cloudstatic.cn/water/tmp_2954a8d96011173a7f2b6baad8cc28317278e368d1393ca6.jpg"
    # picInfo2 = image_to_base64(url_)
    #
    url = "https://cdn.meimeiqushuiyin.cn/imgImprove/tmp_f5648a3c7cc8973b483ee3bb54d23aea.jpg"  # 网络异常
    url = "https://s22-def.ap4r.com/bs2/upload-ylab-stunt-sgp/se/ai_portal_sgp_queue_mmu_txt2img_aiweb/d08b9526-d641-4e04-83d2-27d48e29fa8c/1.png"
    url = "https://s22-def.ap4r.com/bs2/upload-ylab-stunt-sgp/se/ai_portal_sgp_queue_mmu_txt2img_aiweb/9c520b80-efc2-4321-8f0e-f1d34d483ddd/1.png"
    request = BDAITPZSRequest(
        # picInfo=image_to_base64(url),
        original_url=url,
        thumb_url=url
    )

    # from meutils.io.image import image2nowatermark_image
    #
    # api_keys = get_spreadsheet_values(feishu_url=FEISHU_URL, to_dataframe=True)[0]
    # for token in api_keys:
    #     if token:
    #         # request = BDAITPZSRequest(original_url=url, thumb_url=url)
    #         # task = arun(create_task(request, token=token, is_async=False))
    #         # print(task)
    #         arun(image2nowatermark_image(url))
    #         # arun(image2nowatermark_image(url, token=token))

    #
    token = "BAIDUID=FF8BB4BF861992E2BF4A585A37366236:FG=1; BAIDUID_BFESS=FF8BB4BF861992E2BF4A585A37366236:FG=1; BIDUPSID=FF8BB4BF861992E2BF4A585A37366236; BDRCVFR[dG2JNJb_ajR]=mk3SLVN4HKm; userFrom=null; ab_sr=1.0.1_NjY5OWZiZDg5YTJmYTQzNWUyNzU1YjBmN2FlMDFiNjMyOTVhMDE3ZWVlYWY5N2Y2MTg4NGI1MzRmMmVjMjQyZjlhZTU2MmM1NDRlMmU4YzgwMzRiMjUyYTc4ZjY1OTcxZTE4OTA4YTlmMWIwZWUzNTdiMzlhZTRiM2IzYTQ0MjgyMzc2MjQwMGRlYzZlZDhjOTg5Yzg4NWVjMTNiZmVmZQ==; BDRCVFR[-pGxjrCMryR]=mk3SLVN4HKm; H_WISE_SIDS=60273_60360_60623_60664_60678_60684_60700"
    # # token='BAIDUID=F2185337B2F3F85DAB5D2E661BC00C8D:FG=1; BDUSS=hNQkdpbjBOT05EVjRCSFp3TUF-TXFKWHktcFppUGNyd0xZZGp3Mkp6TmVnbFptSVFBQUFBJCQAAAAAAAAAAAEAAACsbZVAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAF71LmZe9S5mOX; BDUSS_BFESS=hNQkdpbjBOT05EVjRCSFp3TUF-TXFKWHktcFppUGNyd0xZZGp3Mkp6TmVnbFptSVFBQUFBJCQAAAAAAAAAAAEAAACsbZVAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAF71LmZe9S5mOX; MAWEBCUID=web_axHhqlIWargVQsXuskqOnMzVXDWQHqQXOpFPkPVSvXFIwTNYaC; PSTM=1716434076; BIDUPSID=5FEB509ECAE0F7843030FD4ECC0800F8; newlogin=1; MCITY=-315%3A; H_WISE_SIDS_BFESS=60360_60674; indexPageSugList=%5B%22%E8%AF%81%E4%BB%B6%E7%85%A7%20%E7%94%B7%22%2C%22%E6%85%88%E7%A6%A7%E5%92%8C%E6%9D%8E%E8%8E%B2%E8%8B%B1%22%2C%22%E7%AE%80%E9%A1%BF%E7%81%AF%E5%85%B7%20%E9%81%A5%E6%8E%A7%E5%99%A8%22%2C%22%E5%88%B0%E5%BD%93%E9%81%93%E5%A3%AB%E9%82%A3%E4%BA%9B%E5%B9%B4%20%E8%B1%86%E7%93%A3%22%2C%22%E9%81%93%E5%A3%AB%20%E5%8D%A1%E9%80%9A%22%2C%22%E9%81%93%E5%A3%AB%22%2C%22%E6%88%91%E5%BD%93%E9%81%93%E5%A3%AB%E9%82%A3%E4%BA%9B%E5%B9%B4%22%2C%22%E8%AF%BB%E6%9E%B6%22%2C%22%E6%9D%B0%E8%AF%BA%E6%AF%94%E5%88%A9%20%E5%AF%B9%E6%AF%94%E7%85%A7%20%E5%8D%95%E8%86%9D%E8%B7%AA%E5%9C%B0%22%5D; BAIDUID_BFESS=F2185337B2F3F85DAB5D2E661BC00C8D:FG=1; BDRCVFR[C0p6oIjvx-c]=I67x6TjHwwYf0; delPer=0; PSINO=5; BDRCVFR[C0sZzZJZb70]=mk3SLVN4HKm; BA_HECTOR=alal0125ah0h0h0l8k8l0l01blu4191jd049m1u; ZFY=C8vEYu7ZbY0zTEITG3A0aWCevri4:BT:B0H1NpNZdQWKM:C; BDORZ=B490B5EBF6F3CD402E515D22BCDA1598; H_PS_PSSID=60360_60674_60683_60694_60572; BCLID=11757829139010941756; BCLID_BFESS=11757829139010941756; BDSFRCVID=pZCOJexroG3bGRntFj9auQWaJgKKv3JTDYLE8yu9T4VNR5DVYtg8EG0Pt_ZYMak-XolpogKKL2OTHm_F_2uxOjjg8UtVJeC6EG0Ptf8g0M5; BDSFRCVID_BFESS=pZCOJexroG3bGRntFj9auQWaJgKKv3JTDYLE8yu9T4VNR5DVYtg8EG0Pt_ZYMak-XolpogKKL2OTHm_F_2uxOjjg8UtVJeC6EG0Ptf8g0M5; H_BDCLCKID_SF=tJkH_KthJIP3e4515Pr_q4tJbq58etJXfaO4Vp7F54nKDp0R0JbI0J0h5p6vaP3m066MaM-ELDOxsMT4QfJJhxuH-qoKel5tQH5MXprN3KJmf-Op2hO0ytDNbHOE2-biW2tH2Mbdax7P_IoG2Mn8M4bb3qOpBtQmJeTxoUJ25DnJhhCGe6KbD6bQDH8Oq-jeHDrKBRbaHJOoDDv8QxRcy4LbKxnxJhoe0H6JXt3LBJnfEpjlbURxqqjy3-OkbfQ9babTQ-tbBp3k8MQXbn7FQfbQ0hOe5RvnMe5a2JK2MJ7JOpvsDxnxy-umQRPH-Rv92DQMVU52QqcqEIQHQT3m5-5bbN3ut6IHJbufoID2fIvDqTrP-trf5DCShUFs5J3dB2Q-XPoO3KOGqpoh5b6b2xKJb-bCb4biQPjNoMbgy4op8P3y0bb2DUA1y4vpX4bOBeTxoUJ2-bjWS-cV-xTqjbIebPRiWTj9QgbLMhQ7tt5W8ncFbT7l5hKpbt-q0x-jLTnhVn0MBCK0MD8xDT8Bj6Pq5fcj2nQXK4_tVTrJabC3qMcJXU6q2bDeQNba0hjXbgcBVRjaLx3HOR3oynD5yl0vWtv4qnctfGPJ-qr4Jlb4otbTeMonDh83Bn6kW4oWHGbyWPJO5hvv8KoO3M7VBUKmDloOW-TB5bbPLUQF5l8-sq0x0bOte-bQXH_E5bj2qRAHoK-h3j; H_BDCLCKID_SF_BFESS=tJkH_KthJIP3e4515Pr_q4tJbq58etJXfaO4Vp7F54nKDp0R0JbI0J0h5p6vaP3m066MaM-ELDOxsMT4QfJJhxuH-qoKel5tQH5MXprN3KJmf-Op2hO0ytDNbHOE2-biW2tH2Mbdax7P_IoG2Mn8M4bb3qOpBtQmJeTxoUJ25DnJhhCGe6KbD6bQDH8Oq-jeHDrKBRbaHJOoDDv8QxRcy4LbKxnxJhoe0H6JXt3LBJnfEpjlbURxqqjy3-OkbfQ9babTQ-tbBp3k8MQXbn7FQfbQ0hOe5RvnMe5a2JK2MJ7JOpvsDxnxy-umQRPH-Rv92DQMVU52QqcqEIQHQT3m5-5bbN3ut6IHJbufoID2fIvDqTrP-trf5DCShUFs5J3dB2Q-XPoO3KOGqpoh5b6b2xKJb-bCb4biQPjNoMbgy4op8P3y0bb2DUA1y4vpX4bOBeTxoUJ2-bjWS-cV-xTqjbIebPRiWTj9QgbLMhQ7tt5W8ncFbT7l5hKpbt-q0x-jLTnhVn0MBCK0MD8xDT8Bj6Pq5fcj2nQXK4_tVTrJabC3qMcJXU6q2bDeQNba0hjXbgcBVRjaLx3HOR3oynD5yl0vWtv4qnctfGPJ-qr4Jlb4otbTeMonDh83Bn6kW4oWHGbyWPJO5hvv8KoO3M7VBUKmDloOW-TB5bbPLUQF5l8-sq0x0bOte-bQXH_E5bj2qRAHoK-h3j; BDRCVFR[dG2JNJb_ajR]=mk3SLVN4HKm; userFrom=null; BDRCVFR[-pGxjrCMryR]=mk3SLVN4HKm; ab_sr=1.0.1_NTY4Y2RjNTU5MWYzM2IyZjY4M2EzMWNjYWM1MWRhYjdlZDhmZTdkZGQ1NGE0MGFkZDU0NWQzNTU1OWJiZDllZmU4YjcyZDIyNTJiNzYxOWQ4OGM1MjQ2ODQzYTM2OGQ3YjdkMzQxZWI4Y2U5ZjQ3ZWZkYTQ2M2I3Yzc2MDkxMjBhZjczODNjMDNmNTAxMzhiZDg2ZmU0MTIwZTUxYWMxNQ==; H_WISE_SIDS=60360_60674_60683_60694_60572'
    # request = BDAITPZSRequest(original_url=url, thumb_url=url)

    url = "https://oss.ffire.cc/files/shuiyin.jpg"
    url = "https://oss.ffire.cc/files/nsfw.jpg"

    request = BDAITPZSRequest(
        type="1",
        # picInfo=image_to_base64(url),
        original_url=url,
        thumb_url=url
    )
    arun(create_task(request, is_async=False, response_format='url'))

    # arun(get_task('cr84hhue1pn8m151of3g', response_format='url'))

    # url = "https://oss.ffire.cc/files/kling_watermark.png"
    # url = "https://cdn.meimeiqushuiyin.cn/2024-10-20/removeBg/tmp_037244e2d6e1ebb8aa6ae3e98584f7b0482fed4ec5ecae98.jpg"
    #
    # request = BaiduImageProcessRequest(
    #     task="matting",
    #     image=url,
    # )
    #
    # arun(image_process(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : url2image
# @Time         : 2024/9/19 08:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

BASE_URL = 'https://url2pic.php127.com'


async def url2image(url, width: int = 1440, type: str = 'jpg'):
    """

    :param url: str or list
    :param width: 640 1080 1440
    :param type: 输出格式 jpg|png|svg|bmp|pdf 默认jpg
    :return:
    """

    payload = {
        'url': url,
        'width': width,
        'type': type,
        'key': 'u2p66eb73ff63b0266eb73ff63b03632',  # todo: 多key轮询

    }
    async with httpx.AsyncClient(base_url=BASE_URL, timeout=60) as client:
        response = await client.post('/api/url2pic', json=payload)
        if response.is_success:
            return response.json()


if __name__ == '__main__':
    url = 'https://www.baidu.com'

    arun(url2image(url, 640))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : raw
# @Time         : 2024/11/20 16:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from meutils.pipe import *
from meutils.schemas.replicate_types import FEISHU_URL, ReplicateRequest, ReplicateResponse
from meutils.config_utils.lark_utils import get_next_token_for_polling

from replicate import Client


@alru_cache(ttl=3600)
async def get_version(model="black-forest-labs/flux-schnell", token: Optional[str] = None):
    replicate = Client(api_token=token)

    model = await replicate.models.async_get(model)
    version = model.latest_version.id
    return version


async def create_task(request: ReplicateRequest):
    token = await get_next_token_for_polling(FEISHU_URL)
    replicate = Client(api_token=token)

    version = await get_version(request.ref, token)
    prediction = await replicate.predictions.async_create(
        version=version,
        input=request.input  # {"prompt": "Watercolor painting of an underwater submarine"})
    )
    return ReplicateResponse(**prediction.dict(), system_fingerprint=token)


async def get_task(task_id, token: Optional[str]=None):
    replicate = Client(api_token=token)

    prediction = await replicate.predictions.async_get(task_id)

    return ReplicateResponse(**prediction.dict())


if __name__ == '__main__':
    request = ReplicateRequest(input={"prompt": "a cat"})
    # task = arun(create_task(request))

    # arun(get_task(task.id))

    arun(get_task("77e6rwn5v5rmc0ck964acn75dc"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : shutdown
# @Time         : 2024/1/9 08:56
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from fastapi import APIRouter, File, UploadFile, Query, Form, Response, Request

router = APIRouter()


@router.on_event("shutdown")
async def shutdown_event():
    # 执行清理操作，关闭事件循环 或者 通知
    await asyncio.get_event_loop().shutdown_asyncgens()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : pipe_utils
# @Time         : 2020/11/12 11:35 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :
import json

from meutils.common import *
from meutils.async_utils.common import arun, aclose, alru_cache, async2sync_generator, close_event_loop, achain


class Pipe(object):
    """I am very like a linux pipe"""

    def __init__(self, function):
        self.function = function
        functools.update_wrapper(self, function)

    def __ror__(self, other):
        return self.function(other)

    # def __call__(self, *args, **kwargs):
    #     return Pipe(lambda x: self.function(x, *args, **kwargs))
    def __call__(self, *args, **kwargs):
        return Pipe(
            lambda iterable, *args2, **kwargs2: self.function(
                iterable, *args, *args2, **kwargs, **kwargs2
            )
        )


########### 常用管道函数
# 进度条
xtqdm = Pipe(lambda iterable, desc=None: tqdm(iterable, desc))

# base types
xtuple, xlist, xset = Pipe(tuple), Pipe(list), Pipe(set)
xarray = Pipe(lambda x, decimals=None: np.array(x) if decimals is None else np.round(np.array(x), decimals))

# 高阶函数
xmap = Pipe(lambda iterable, func: map(func, iterable))
xmap_ = Pipe(lambda iterable, func: list(map(func, iterable)))

xzip = Pipe(lambda iterable, ismax=None: zip(*iterable) if ismax is None else itertools.zip_longest(*iterable))
xzip_ = Pipe(
    lambda iterable, ismax=None: list(zip(*iterable)) if ismax is None else list(itertools.zip_longest(*iterable)))

xreduce = Pipe(lambda iterable, func: reduce(func, iterable))

xfilter = Pipe(lambda iterable, func=None: filter(func, iterable))
xfilter_ = Pipe(lambda iterable, func=None: list(filter(func, iterable)))
# xtake = Pipe(lambda iterable, func=None: itertools.takewhile(func, iterable))
# xtake_ = Pipe(lambda iterable, func=None: list(itertools.takewhile(func, iterable)))
xdrop = Pipe(lambda iterable, func=None: itertools.dropwhile(func, iterable))
xdrop_ = Pipe(lambda iterable, func=None: list(itertools.dropwhile(func, iterable)))

# itertools: https://blog.csdn.net/weixin_43193719/article/details/87536371
xchain = Pipe(lambda iterable: itertools.chain(*iterable))
xchain_ = Pipe(lambda iterable: list(itertools.chain(*iterable)))

xenumerate = Pipe(lambda iterable, start=0: enumerate(iterable, start))
xenumerate_ = Pipe(lambda iterable, start=0: list(enumerate(iterable, start)))

xshuffle = Pipe(lambda l, n_samples=None: sklearn.utils.shuffle(l, n_samples=n_samples))

# dateframe dfs
"""多个df: 
dfs = (
    Path('.').glob('demo*.txt') | xmap(lambda p: pd.read_csv(p, chunksize=2, names=['id'])) | xchain
)
"""
xconcat4df = Pipe(
    lambda dfs, convert_func=lambda x: x, axis=0: pd.concat(map(convert_func, dfs), ignore_index=True, axis=axis))

# str
xjoin = Pipe(lambda chars, sep=' ': sep.join(map(str, chars)))
xsort = Pipe(lambda iterable, reverse=False: sorted(iterable, reverse=reverse))
xitemgetter = Pipe(lambda keys, dic: operator.itemgetter(*keys)(dic))
xstartswith = Pipe(lambda iterable, prefix=('_', '__', '.'): filter(lambda p: p.startswith(prefix)))
xendswith = Pipe(lambda iterable, suffix=('_', '__', '.'): filter(lambda p: p.startswith(suffix)))

# 统计词频
xCounter = Pipe(lambda iterable: Counter(iterable))


@Pipe
def xprint(iterable, end='', desc='Print'):
    close_event_loop()
    iterable = async2sync_generator(iterable)
    if desc:
        iterable = tqdm(iterable, desc=desc)

    for i in iterable:
        print(i, end=end)


@Pipe
def xCounterUpdate(iterable, counter: Counter = None):
    """[['w1', 'w2'], ...]"""
    counter = counter or Counter()
    for i in iterable:
        counter.update(i)
    return counter


# operator: 排序、取多个值     https://blog.csdn.net/u010339879/article/details/98304292
# operator.itemgetter(*keys)(dic)


@Pipe
def xgetitem(iterable, index=0):
    """

    @param iterable: [(0, 1), (1, 2)] | xgetitem
    @param index:
    @return:
    """
    for i in iterable:
        yield operator.getitem(i, index)


# dict
@Pipe
def xchain_dict(iterable: List[Dict]):
    dic = {}
    for d in iterable:
        dic.update(d)
    return dic


@Pipe
def xDictValues(keys, dic: dict, default=None):
    return tuple(dic.get(k, default) for k in keys)


@Pipe
def xDictRemove(keys, dic: dict):
    for k in keys:
        if k in dic:
            del dic[k]


# np
xstack = Pipe(lambda iterable, axis=0: np.stack(iterable, axis=axis))
xrow_stack = Pipe(lambda iterable, axis=0: np.row_stack(iterable))

# 调试用
xnext = Pipe(lambda ls: iter(ls).__next__())


@Pipe
def xUnique(iterable, keep_order=True):
    if keep_order:
        return list(OrderedDict.fromkeys(iterable))  # 移除列表中的重复元素(保持有序)
    else:
        return list(set(iterable))


@Pipe
def xUnique_plus(iterable, key_fn: Callable = None):
    from joblib.hashing import hash
    hash_dict = {}

    for element in iterable:
        if key_fn:
            key = key_fn(element)
        else:
            key = element
        hash_dict[hash(key)] = element
    return list(hash_dict.values())


@Pipe
def xBloomFilter(iterable, bloom=None):
    """
        def bloom_add(iterable):
        bloom = pkl_load('bloom')
        for i in iterable:
            bloom.add(i)
        pkl_dump(bloom, 'bloom')
    """
    from pybloom_live import ScalableBloomFilter, BloomFilter
    bloom = bloom or ScalableBloomFilter(10 ** 6)
    for i in iterable:
        bloom.add(i)
    return bloom


@Pipe
def xbar4iter(iterable: list, func, batch_size=1, show_bar=True) -> list:
    """func 入参出参都是 list"""
    l = []
    _ = iterable | xgroup(batch_size)
    if show_bar: _ = tqdm(_)
    for i in _:
        l += func(i)
    return l


@Pipe
def xgroup(iterable, step=3, bins=None):
    n = len(iterable)
    if bins:
        step = max(n // bins, 1)

    _ = [iterable[idx: idx + step] for idx in range(0, n, step)]

    # if len(bins) > bins:  # todo: 确保正确的bins

    return _


@Pipe
def xsections(iterable, batch_size=3):
    def generator():
        yield from iterable

    g = generator()

    while True:
        group = list(itertools.islice(g, batch_size))
        if not group:
            break
        yield group


@Pipe
def xHashBins(ls, bins=3):
    """hash分组"""
    dic = {}
    for v in ls:
        dic.setdefault(murmurhash(v, bins=bins), []).append(v)
    return list(dic.values())


@Pipe
def xJobs(iterable, func, n_jobs=3):
    """支持匿名函数"""
    if n_jobs > 1:
        delayed_function = joblib.delayed(func)
        return joblib.Parallel(n_jobs=n_jobs)(delayed_function(arg) for arg in iterable)
        # yield from joblib.Parallel(n_jobs=n_jobs)(delayed_function(arg) for arg in iterable)
    else:
        return list(map(func, iterable))


@Pipe
def xThreadPoolExecutor(iterable, func, max_workers=5, desc="Processing", unit="it"):
    total = len(iterable) if hasattr(iterable, '__len__') else None
    if total == 1:
        max_workers = total

    if max_workers > 1:
        with ThreadPoolExecutor(max_workers) as pool, tqdm(total=total, desc=desc, unit=unit) as pbar:
            for i in pool.map(func, iterable):
                yield i
                pbar.update()

    else:
        yield from map(func, iterable)


@Pipe
def xProcessPoolExecutor(iterable, func, max_workers=5, desc="Processing", unit="it"):
    total = len(iterable) if hasattr(iterable, '__len__') else None
    if total == 1:
        max_workers = total

    if max_workers > 1:
        with ProcessPoolExecutor(max_workers) as pool, tqdm(total=total, desc=desc, unit=unit) as pbar:
            for i in pool.map(func, iterable):
                yield i
                pbar.update()

    else:
        yield from map(func, iterable)


# 异步
@Pipe
def xAsyncio(tasks, return_exceptions=False):
    """为了从异步方式获益，一个应用程序需要有经常被 I/O 阻塞的任务，并且没有太多 CPU 工作。Web 应用程序通常非常适合，特别是当它们需要处理大量客户端请求时。
        import nest_asyncio
        nest_asyncio.apply()

        from asgiref.sync import sync_to_async

        @sync_to_async(thread_sensitive=False)
        def hello():
            time.sleep(1)
            return time.ctime()

        [hello() for _ in range(10)] | xAsyncio

    asyncio.run(asyncio.wait(tasks))  # 3.8+
    """
    loop = asyncio.get_event_loop()
    _ = asyncio.gather(*tasks, return_exceptions=return_exceptions)  # asyncio.wait(tasks)

    return loop.run_until_complete(_)


@Pipe
def xsse_parser(iterable, prefix: str = 'data:', skip_strings: Optional[list] = None):
    l = []
    skip_strings = skip_strings or []
    for line in iterable:
        if (line := line.lstrip()) and line.startswith(prefix):
            if any(s in line for s in skip_strings):
                continue
            try:
                _ = json.loads(line[len(prefix):])
                l.append(_)
            except Exception as e:
                logger.debug(f'error: {e}\nline: {line}')
    return l


if __name__ == '__main__':
    # @Pipe
    # def xfunc1(x):
    #     _ = x.split()
    #     print(_)
    #     return _
    #
    #
    # @Pipe
    # def xfunc2(x):
    #     _ = '>>'.join(x)
    #     print(_)
    #     return _
    #
    #
    # def wrapper(func):
    #     @functools.wraps(func)
    #     def wrapped(*args, **kwargs):
    #         logger.patch(lambda r: r.update(name='__file__', function=func.__name__)).info("Wrapped!")
    #         return func(*args, **kwargs)
    #
    #     return wrapped
    #
    #
    # # log = 'I am very like a linux pipe' | xfunc1 | xfunc2
    # # logger.info(log)
    # #
    # # logger = logger.patch(lambda r: r.update(name=__file__, function=''))  # main:module
    # # logger.info(log)
    # #
    # # # logger = logger.patch(wrapper(lambda x: ''))
    # # # logger.info(log)
    #
    # ['aaaa', 'vvvvv'] | xprint
    #
    #
    #
    # def single(a):
    #     """ 定义一个简单的函数  """
    #     time.sleep(1)
    #
    #
    # with timer('并行'):
    #     range(10) | xJobs(single)

    # s = pd.DataFrame(range(10), columns=['a']).a
    # idxs = np.where(s.isin((3, 6)))[0]
    # ss = pd.cut(dd.a, [-np.inf, *idxs, np.inf])
    # ss.groupby(ss).groups

    # print(range(10) | xThreadPoolExecutor(print, 1) | xlist)
    # print(range(10) | xThreadPoolExecutor(print, 2) | xlist)

    # print(range(10) | xProcessPoolExecutor(print, 1) | xlist)
    # print(range(10) | xProcessPoolExecutor(print, 2) | xlist)

    sse = """
    data: {"type":"step","msg":"努力生成中...","code":200}
data: {"type":"progress","value":0.08333333333333333,"code":200}
data: {"type":"progress","value":0.16666666666666666,"code":200}
data: {"type":"progress","value":0.25,"code":200}
data: {"type":"progress","value":0.3333333333333333,"code":200}
data: {"type":"progress","value":0.4166666666666667,"code":200}
data: {"type":"progress","value":0.5,"code":200}
data: {"type":"removewatermark","id":"c0d1e312-b374-11ef-a22c-6a452a83f9ec","imageUrl":"https://hunyuan-prod-1258344703.cos.ap-guangzhou.myqcloud.com/removewatermark/f55df3ef029c13eba0832a11f77d03ad/20241206095236h0_d589b0620869d87ad340b4b4ed68b4ec.png?q-sign-algorithm=sha1\u0026q-ak=AKID0qSq0xJRL7h3A4nIYJFrFOJ1VlnbIm26\u0026q-sign-time=1733449957;1764985957\u0026q-key-time=1733449957;1764985957\u0026q-header-list=host\u0026q-url-param-list=\u0026q-signature=7bbec24fe87917896f662f65b80320a4a3fb8569","source":"inspiration","code":200,"thumbnailUrl":"https://hunyuan-prod-1258344703.cos.ap-guangzhou.myqcloud.com/removewatermark/f55df3ef029c13eba0832a11f77d03ad/20241206095236h0_d589b0620869d87ad340b4b4ed68b4ec.png?q-sign-algorithm=sha1\u0026q-ak=AKID0qSq0xJRL7h3A4nIYJFrFOJ1VlnbIm26\u0026q-sign-time=1733449957;1764985957\u0026q-key-time=1733449957;1764985957\u0026q-header-list=host\u0026q-url-param-list=\u0026q-signature=7bbec24fe87917896f662f65b80320a4a3fb8569\u0026imageMogr2/strip/format/jpg/size-limit/500k!/ignore-error/1"}
data: [TRACEID:8a50e3687e26e03097e507c076fe7145]
data: [DONE]"""

    # print()

    # sse.splitlines() | xsse_parser(skip_strings=['DONE', 'TRACEID']) | xprint


    a = [[{"a":1}]] | xgroup(2)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : demo
# @Time         : 2020-02-14 11:52
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  :
# https://github.com/milvus-io/docs/blob/master/site/en/guides/milvus_operation.md#createdrop-indexes-in-a-collection
# https://github.com/milvus-io/pymilvus/tree/master/examples/indexes
# https://raw.githubusercontent.com/milvus-io/pymilvus/0.7.0/examples/example.py
# TODO:
# m.milvus.preload_table

from meutils.pipe import *
from milvus import Milvus, DataType
import copy

# IndexType.

# from milvus.client.exceptions import CollectionNotExistException

"""
client.drop_index
client.get_config
client.list_id_in_segment
client.load_collection???
"""


class Collection(object):

    def __init__(self, name=None, client=None):
        self.name = name
        self.client = client
        self.count_entities = self.count
        self.count_documents = self.count
        self.vector_name = self.get_vec_field_name()

    def __str__(self):
        has_collection = self.client.has_collection(self.name)
        if not has_collection:
            logger.warning(f"{self.name}  doesn't exist")
        return f"Collection({self.name})"

    def batch_insert(self, df_entity: pd.DataFrame, batch_size=100000):
        """

        :param df_entity: id, sid, vec, part 与 collection 字段一致
        :param batch_size:
        :return:
        """
        entity_names = [_['name'] for _ in self.collection_info['fields']]
        logger.warning(f"EntityNames: {entity_names}")

        # 分区
        df_entity = df_entity.reset_index(drop=True)
        n = len(df_entity)
        num_part = n // batch_size + 1 if n % batch_size else n // batch_size

        ids = []
        for i in tqdm(range(num_part), desc='BatchInsert'):
            df = df_entity.iloc[i * batch_size:(i + 1) * batch_size, :]
            entities = []
            for record in self.collection_info['fields']:
                entities.append({
                    'name': record['name'],
                    'type': record['type'],
                    'values': df[record['name']].values
                })

            ids += self.client.insert(self.name, entities, ids=df['id'] if 'id' in df else None)
            time.sleep(3)
        return ids

    def search(
            self, vectors=np.random.random((1, 256)),
            topk=10,
            nprobe=1,
            scalar_list: List[dict] = None):
        q = self.get_search_query(vectors, topk, nprobe, scalar_list)
        entities = self.client.search(self.name, q)[0]

        return entities  # todo: 增加阈值过滤

        # entities = ann.client.search("demo", query_hybrid)[0]
        # id2score = dict(zip(entities.ids, entities.distances))

        # docs = mongo_collection.find({"xindaoid": {'$in': entities.ids}})
        # df = pd.DataFrame(list(docs)).drop(['_id', 'category_', 'vector'], 1)
        # df['distance'] = df['xindaoid'].map(id2score)

    def batch_search(self, vectors=np.random.random((1, 256)), topk=10, nprobe=1,
                     scalar_list: List[dict] = None):
        q = self.get_search_query(vectors, topk, nprobe, scalar_list)
        entities = self.client.search(self.name, q)
        return entities

    def get_entity_by_id(self, ids, fields=None):
        return self.client.get_entity_by_id(self.name, ids, fields)

    def delete_entity_by_id(self, ids):
        self.client.delete_entity_by_id(self.name, ids)

    @property
    def count(self):
        return self.client.count_entities(self.name)

    @property
    def collection_info(self):
        return self.client.get_collection_info(self.name)

    @property
    def collection_stats(self):
        return self.client.get_collection_stats(self.name)

    def get_vec_field_name(self):
        fields = self.collection_info['fields']
        vec_field = [_ for _ in fields if str(_.get('type', '')).__contains__('VECTOR')][0]
        return vec_field['name']

    def get_search_query(self, vectors, topk=10, nprobe=1, scalar_list: List[dict] = None):
        """
        ann.demo.search(np.random.random((1, 10)), scalar_list=[{'term': {'scalar': [1,2,3,4]}}])
        :param vectors:
        :param topk:
        :param nprobe:
        :param scalar_list:
        :return:
        """
        q = {
            "bool": {
                "must": [
                    {
                        "vector": {
                            self.vector_name: {
                                "topk": topk,
                                "query": vectors,
                                "metric_type": "IP",
                                "params": {
                                    "nprobe": nprobe
                                }
                            }
                        }
                    },
                ]
            }
        }
        if scalar_list is not None:  # {"term": {"标量字段": [1,2,3]}}
            for _ in scalar_list:
                q['bool']['must'].append(_)
        return q


class ANN(object):

    def __init__(self, host='HOST', port='19530', handler="GRPC", pool="SingletonThread", show_info=False):
        self.host = host
        self.client = Milvus(host, port, handler=handler, pool=pool, pool_size=32)  # 线程池

        if show_info:
            logger.info(
                {
                    "ClientVersion": self.client.client_version(),
                    "ServerVersion": self.client.server_version()
                }
            )

    def __getattr__(self, collection_name) -> Collection:
        return Collection(collection_name, self.client)

    def create_collection(self, collection_name, fields, auto_id=True, segment_row_limit=4096, overwrite=True):
        """

        :param collection_name:
        :param fields: # type: BOOL INT32 INT64 FLOAT BINARY_VECTOR FLOAT_VECTOR
            fields = [
                {
                    "name": "scalar",
                    "type": 'INT32',
                    "params": {},
                    "indexes": [{}]
                },
                {
                    "name": "vector",
                    "type": 'FLOAT_VECTOR',
                    "params": {"dim": 768},
                    "indexes": [{"index_type": 'IVF_FLAT', 'metric_type': 'IP', 'params': {'nlist': 1024}, 'index_file_size': 1024}]
                }
            ]
        # index_file_size不确定放在哪生效
        :param auto_id:
        :param segment_row_limit: range 4096 ~ 4194304
        :return:
        """
        fields = copy.deepcopy(fields)  # fields[:]

        if self.client.has_collection(collection_name):
            if overwrite:
                logger.warning(f"{collection_name} already exists! to drop.")
                self.client.drop_collection(collection_name, timeout=300)
            else:
                return f"{collection_name} already exists!"

        vec_field = [_ for _ in fields if _.get('type', '').__contains__('VECTOR')][0]
        # assert len(vec_fields) > 0, "至少有一个矢量"

        for _ in fields:
            if 'type' in _:
                _['type'] = DataType.__getattr__(_['type'])

        collection_param = {
            "fields": fields,
            "auto_id": auto_id,
            "segment_row_limit": segment_row_limit,
        }

        # collection vector index
        self.client.create_collection(collection_name, fields=collection_param)

        self.client.create_index(collection_name, vec_field['name'], vec_field['indexes'][0])

        logger.info(f"{self.client.get_collection_info(collection_name)}")

    @property
    def collection_names(self):
        return self.client.list_collections()

    def __create_index(self, collection_name, field_name, index_type='IVF_FLAT', metric_type='IP', index_params=None):

        if index_params is None:
            index_params = {'nlist': 1024}

        params = {
            'index_type': index_type,
            # 'index_file_size': 1024, # TODO: 不确定放在哪生效
            'params': index_params,

            'metric_type': metric_type,
        }
        self.client.create_index(collection_name, field_name, params)  # field_name='embedding'


if __name__ == '__main__':
    ann = ANN('HOST', show_info=True)
    fields = [
        {
            "name": "scalar",
            "type": 'INT32',
            "params": {},
            "indexes": [{}]
        },
        {
            "name": "vector",
            "type": 'FLOAT_VECTOR',
            "params": {"dim": 256},
            "indexes": [
                {"index_type": 'IVF_FLAT', 'metric_type': 'IP', 'params': {'nlist': 1024}, 'index_file_size': 1024}]
        }
    ]
    ann.create_collection('demo', fields)
    print(ann.demo)
    print(ann.demo.collection_info)
    print(ann.demo.vector_name)
    # print(ann.demo.collection_stats)

    # df列名必须与fields一致
    df = pd.DataFrame(enumerate('abcdefgh'), columns=['scalar', 'sid']).assign(
        vector=np.random.random((8, 256)).tolist())

    ann.demo.batch_insert(df)
